define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/dom",
        "dojo/dom-style",
        "dojo/Evented",
        "dojo/on",
        "dijit/focus",
        "dijit/Tooltip",
        "com/sas/dm/datadirector/util/SharedProperties",
        "com/sas/dm/datadirector/widgets/MessageBox",
        "com/sas/dm/datadirector/models/servers/JdbcServer",
        "com/sas/dm/datadirector/models/servers/FederationJdbcServer",
        "com/sas/dm/datadirector/models/servers/OracleJdbcServer",
        "com/sas/dm/datadirector/models/servers/TeradataJdbcServer",
        "com/sas/dm/datadirector/views/presentation/DatabaseConfigurationDialogPM",
        "com/sas/dm/common/util/UIUtils",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/common/util/BindingUtils",
        "com/sas/dm/common/components/TextFieldDialog",
        "com/sas/dm/common/components/ConfirmationDialog",
        "com/sas/dm/common/components/base/util/ContextUtils",
        "com/sas/dm/common/components/base/_ComponentBase",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], function(declare, _lang, dom, domStyle, Evented, on, focus, Tooltip, SharedProperties, MessageBox, JdbcServer, FederationJdbcServer, OracleJdbcServer, TeradataJdbcServer, DatabaseConfigurationDialogPM, UIUtils, StringUtils, BindingUtils, TextFieldDialog, ConfirmationDialog, ContextUtils, _ComponentBase, ResourceBundle) {
	var DatabaseConfigurationDialog = declare([_ComponentBase], {
		_pm:null,
		_pmBound:false,
		_controller:null,
		_showAfterDownload:false,
		_showContainer:null,
		_initialShow:false,
		_errorDetailsDialog:null,
		_jdbcServerWatcher:null,
		_userNameWatcher:null,
		_passwordWatcher:null,
		_errorDetailsWatcher:null,
		_lastTypeSelection:"",
		_pendingSaveCalls:0,
		_dialogHasErrors:false,
		_ignoreChange:false,
		_isConnStrEdited:false,
		
		_setContent:function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
			
			cont = cont.replace(/([\w\d_]+)_\$I18N\$/g,function(match, p1){
				console.debug(p1);
				return ResourceBundle[p1];
			});
			return this.inherited(arguments, [cont, isFakeContent]);
		},

		/*Override*/
		__onDownloadComplete:function() {
			this.inherited(arguments);
			
			if (this._showAfterDownload) {
				this._showAfterDownload = false;
				this._internalShow();
			}
		},

		/*Override*/
		// if pmClass were passed in as a parameter to the constructor, we would not have
		// to override _getPresentationModelInstance, but since we're supplying the implementation
		// directly, this is the place to do it.
		_getPresentationModelInstance:function() {
			if (this._pm == null){
				// this class doesn't need to be context aware, but it does need a reference
				// to the most local context to pass to the PM, so we use ContextUtils.FindContext
				var context = ContextUtils.FindContext(this);
				this._pm = new DatabaseConfigurationDialogPM({"context":context});
			}
			return this._pm;
		},
		
		/*Override*/
		// this is where we're actually binding any properties exposed in the PM to the view
		_bindPresentationModel:function() {
			var pm = this._getPresentationModelInstance();
			if (pm == null){
				console.error("No PM defined in DatabaseConfigurationDialog.js");
				return;
			}
			
			pm.set("controller", this._controller);
			this._jdbcServerWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onJdbcServerChange), pm, "jdbcServer");
			this._userNameWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onUserNameChange), pm, "userName");
			this._passwordWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onPasswordChange), pm, "password");
			this._errorDetailsWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onErrorDetailsChange), pm, "errorDetails");
			this._pmBound = true;
		},
		
		_onJdbcServerChange:function(value/*JdbcServer*/) {
			var nameField = this.findChildByID("DatabaseConfigurationDialogName");
			var descriptionField = this.findChildByID("DatabaseConfigurationDialogDescription");
			var typeSelector = this.findChildByID("DatabaseConfigurationDialogType");
			var driverField = this.findChildByID("DatabaseConfigurationDialogDriver");
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var hostAsterisk = dom.byId("DatabaseConfigurationDialogHostAsterisk" + this._getIDSuffix());
			var portLabel = dom.byId("DatabaseConfigurationDialogPortLabel" + this._getIDSuffix());
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var portAsterisk = dom.byId("DatabaseConfigurationDialogPortAsterisk" + this._getIDSuffix());
			var dbNameField = this.findChildByID("DatabaseConfigurationDialogDBName");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var connStrEdit = this.findChildByID("DatabaseConfigurationDialogConnectStringEdit");
			var connStrReset = this.findChildByID("DatabaseConfigurationDialogConnectStringReset");
			var sidEntry = dom.byId("DatabaseConfigurationDialogSIDEntry" + this._getIDSuffix());
			var sidAsterisk = dom.byId("DatabaseConfigurationDialogSIDAsterisk" + this._getIDSuffix());
			var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
			var connMgrEntry = dom.byId("DatabaseConfigurationDialogConnMgrEntry" + this._getIDSuffix());
			var connMgrAsterisk = dom.byId("DatabaseConfigurationDialogConnMgrAsterisk" + this._getIDSuffix());
			var connMgrField = this.findChildByID("DatabaseConfigurationDialogConnMgr");
			var dsnEntry = dom.byId("DatabaseConfigurationDialogDsnEntry" + this._getIDSuffix());
			var dsnAsterisk = dom.byId("DatabaseConfigurationDialogDsnAsterisk" + this._getIDSuffix());
			var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
			if (nameField == null) {
				return;  // UI components not available yet
			}
			if(value == null){
				// trick the UI into rendering the default state
				value = {"get":function(){return null}};
			}
			this._ignoreChange = true;
			
			nameField.set("value", value.get("name"));
			descriptionField.set("value", value.get("description"));
			
			var driverClass = value.get("driverClassName");
			var connStr = value.get("connectionString");
			domStyle.set(sidEntry, "display", "none");
			sidField.set("disabled", true);
			domStyle.set(connMgrEntry, "display", "none");
			connMgrField.set("disabled", true);
			domStyle.set(dsnEntry, "display", "none");
			dsnField.set("disabled", true);
			var serverProperties = value.get("properties") || {};
			if (driverClass == null || driverClass.length == 0 || driverClass == SharedProperties.jdbcConfiguration_oracledriver) {  // Oracle is the default
				this._lastTypeSelection = "oracle";
				typeSelector.set("value", "oracle");
				driverField.set("value", SharedProperties.jdbcConfiguration_oracledriver);
				driverField.set("disabled", true);
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_port_label;
				domStyle.set(portField.domNode, "display", "");
				domStyle.set(dbNameField.domNode, "display", "none");
				domStyle.set(sidEntry, "display", "block");
				dbNameField.set("disabled", true);  // Prevent validation
				if (connStr == null || connStr.length == 0) {
					hostField.set("disabled", false);
					hostField.set("value", value.get("hostName"));
					domStyle.set(hostAsterisk, "visibility", "visible");
					portField.set("disabled", false);
					portField.set("value", value.get("port"));
					domStyle.set(portAsterisk, "visibility", "visible");
					sidField.set("disabled", false);
					sidField.set("value", serverProperties.service_name || "");
					domStyle.set(sidAsterisk, "visibility", "visible");
					connStrField.set("disabled", true);
					connStrField.set("value", this._buildSampleOracleConnectString(value.get("hostName"), value.get("port"), serverProperties.service_name));
					connStrEdit.set("disabled", false);
					connStrReset.set("disabled", true);
					this._isConnStrEdited = false;
				}
				else {
					hostField.set("disabled", true);
					hostField.set("value", "");
					domStyle.set(hostAsterisk, "visibility", "hidden");
					portField.set("disabled", true);
					portField.set("value", "");
					domStyle.set(portAsterisk, "visibility", "hidden");
					sidField.set("disabled", true);
					sidField.set("value", "");
					domStyle.set(sidAsterisk, "visibility", "hidden");
					connStrField.set("disabled", false);
					connStrField.set("value", connStr);
					connStrEdit.set("disabled", true);
					connStrReset.set("disabled", false);
					this._isConnStrEdited = true;
				}
			}
			else if (driverClass == SharedProperties.jdbcConfiguration_teradatadriver) {
				this._lastTypeSelection = "teradata";
				typeSelector.set("value", "teradata");
				driverField.set("value", driverClass);
				driverField.set("disabled", true);
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_dbname_label;
				domStyle.set(portField.domNode, "display", "none");
				portField.set("disabled", true);  // Prevent validation
				domStyle.set(dbNameField.domNode, "display", "");
				domStyle.set(connMgrEntry, "display", "block");
				domStyle.set(connMgrAsterisk, "visibility", "hidden");
				connMgrField.set("disabled", false);
				connMgrField.set("required", false);
				connMgrField.set("value", value.get("connectionManager"));
				if (connStr == null || connStr.length == 0) {
					hostField.set("disabled", false);
					hostField.set("value", value.get("hostName"));
					domStyle.set(hostAsterisk, "visibility", "visible");
					dbNameField.set("disabled", false);
					dbNameField.set("value", value.get("databaseName"));
					domStyle.set(portAsterisk, "visibility", "visible");
					connStrField.set("disabled", true);
					connStrField.set("value", this._buildSampleTeradataConnectString(value.get("hostName"), value.get("databaseName")));
					connStrEdit.set("disabled", false);
					connStrReset.set("disabled", true);
					this._isConnStrEdited = false;
				}
				else {
					hostField.set("disabled", true);
					hostField.set("value", "");
					domStyle.set(hostAsterisk, "visibility", "hidden");
					dbNameField.set("disabled", true);
					dbNameField.set("value", "");
					domStyle.set(portAsterisk, "visibility", "hidden");
					connStrField.set("disabled", false);
					connStrField.set("value", connStr);
					connStrEdit.set("disabled", true);
					connStrReset.set("disabled", false);
					this._isConnStrEdited = true;
				}
			}
			else if (driverClass == SharedProperties.jdbcConfiguration_federationdriver) {
				this._lastTypeSelection = "federation";
				typeSelector.set("value", "federation");
				driverField.set("value", SharedProperties.jdbcConfiguration_federationdriver);
				driverField.set("disabled", true);
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_port_label;
				domStyle.set(portField.domNode, "display", "");
				domStyle.set(dbNameField.domNode, "display", "none");
				domStyle.set(dsnEntry, "display", "block");
				dbNameField.set("disabled", true);  // Prevent validation
				if (connStr == null || connStr.length == 0) {
					hostField.set("disabled", false);
					hostField.set("value", value.get("hostName"));
					domStyle.set(hostAsterisk, "visibility", "visible");
					portField.set("disabled", false);
					portField.set("value", value.get("port"));
					domStyle.set(portAsterisk, "visibility", "visible");
					dsnField.set("disabled", false);
					dsnField.set("value", serverProperties.dsn || "");
					domStyle.set(dsnAsterisk, "visibility", "visible");
					connStrField.set("disabled", true);
					connStrField.set("value", this._buildSampleFederationConnectString(value.get("hostName"), value.get("port"), serverProperties.dsn));
					connStrEdit.set("disabled", false);
					connStrReset.set("disabled", true);
					this._isConnStrEdited = false;
				}
				else {
					hostField.set("disabled", true);
					hostField.set("value", "");
					domStyle.set(hostAsterisk, "visibility", "hidden");
					portField.set("disabled", true);
					portField.set("value", "");
					domStyle.set(portAsterisk, "visibility", "hidden");
					dsnField.set("disabled", true);
					dsnField.set("value", "");
					domStyle.set(dsnAsterisk, "visibility", "hidden");
					connStrField.set("disabled", false);
					connStrField.set("value", connStr);
					connStrEdit.set("disabled", true);
					connStrReset.set("disabled", false);
					this._isConnStrEdited = true;
				}
			}
			else {
				this._lastTypeSelection = "other";
				typeSelector.set("value", "other");
				driverField.set("value", driverClass);
				driverField.set("disabled", false);
				hostField.set("value", "");
				hostField.set("disabled", true);
				domStyle.set(hostAsterisk, "visibility", "hidden");
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_port_label;
				domStyle.set(portField.domNode, "display", "");
				domStyle.set(dbNameField.domNode, "display", "none");
				dbNameField.set("disabled", true);  // Prevent validation
				portField.set("disabled", true);
				portField.set("value", "");
				domStyle.set(portAsterisk, "visibility", "hidden");
				domStyle.set(connMgrEntry, "display", "block");
				domStyle.set(connMgrAsterisk, "visibility", "hidden");
				connMgrField.set("disabled", false);
				connMgrField.set("required", false);
				connMgrField.set("value", value.get("connectionManager"));
				connStrField.set("disabled", false);
				connStrField.set("value", connStr);
				connStrEdit.set("disabled", true);
				connStrReset.set("disabled", true);
				this._isConnStrEdited = true;
			}
			
			this._ignoreChange = false;
		},
		
		setJdbcServer:function(server/*JdbcServer*/) {
			var pm = this._getPresentationModelInstance();
			pm.set("jdbcServer", server);
		},
		
		getJdbcServer:function() {
			var pm = this._getPresentationModelInstance();
			return pm.get("jdbcServer");
		},
		
		_onUserNameChange:function(value/*String*/) {
			var userField = this.findChildByID("DatabaseConfigurationDialogUserID");
			if (userField != null) {
				userField.set("value", value);
			}
		},
		
		_onPasswordChange:function(value/*String*/) {
			var passwordField = this.findChildByID("DatabaseConfigurationDialogPassword");
			if (passwordField != null) {
				passwordField.set("value", value);
			}
		},
		
		_onTypeChange:function(evt) {
			var typeSelector = this.findChildByID("DatabaseConfigurationDialogType");
			if (typeSelector == null) {
				return;
			}
			
			var selType = typeSelector.get("value");
			if (selType == this._lastTypeSelection) {
				return;
			}
			
			var driverField = this.findChildByID("DatabaseConfigurationDialogDriver");
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var hostAsterisk = dom.byId("DatabaseConfigurationDialogHostAsterisk" + this._getIDSuffix());
			var portLabel = dom.byId("DatabaseConfigurationDialogPortLabel" + this._getIDSuffix());
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var portAsterisk = dom.byId("DatabaseConfigurationDialogPortAsterisk" + this._getIDSuffix());
			var dbNameField = this.findChildByID("DatabaseConfigurationDialogDBName");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var connStrEdit = this.findChildByID("DatabaseConfigurationDialogConnectStringEdit");
			var connStrReset = this.findChildByID("DatabaseConfigurationDialogConnectStringReset");
			var sidEntry = dom.byId("DatabaseConfigurationDialogSIDEntry" + this._getIDSuffix());
			var sidAsterisk = dom.byId("DatabaseConfigurationDialogSIDAsterisk" + this._getIDSuffix());
			var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
			var dsnEntry = dom.byId("DatabaseConfigurationDialogDsnEntry" + this._getIDSuffix());
			var dsnAsterisk = dom.byId("DatabaseConfigurationDialogDsnAsterisk" + this._getIDSuffix());
			var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
			var userField = this.findChildByID("DatabaseConfigurationDialogUserID");
			var passwordField = this.findChildByID("DatabaseConfigurationDialogPassword");
			var connMgrEntry = dom.byId("DatabaseConfigurationDialogConnMgrEntry" + this._getIDSuffix());
			var connMgrAsterisk = dom.byId("DatabaseConfigurationDialogConnMgrAsterisk" + this._getIDSuffix());
			var connMgrField = this.findChildByID("DatabaseConfigurationDialogConnMgr");
			
			this._ignoreChange = true;
			domStyle.set(sidEntry, "display", "none");
			sidField.set("disabled", true);
			domStyle.set(connMgrEntry, "display", "none");
			connMgrField.set("disabled", true);
			domStyle.set(dsnEntry, "display", "none");
			dsnField.set("disabled", true);
			if (selType == "oracle") {
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_port_label;
				domStyle.set(portField.domNode, "display", "");
				domStyle.set(dbNameField.domNode, "display", "none");
				dbNameField.set("disabled", true);  // Prevent validation
				driverField.set("value", SharedProperties.jdbcConfiguration_oracledriver);
				driverField.set("disabled", true);
				hostField.set("disabled", false);
				hostField.set("value", "");
				domStyle.set(hostAsterisk, "visibility", "visible");
				portField.set("disabled", false);
				portField.set("value", "");
				domStyle.set(portAsterisk, "visibility", "visible");
				sidField.set("disabled", false);
				sidField.set("value",  "");
				domStyle.set(sidEntry, "display", "block");
				domStyle.set(sidAsterisk, "visibility", "visible");
				connStrField.set("disabled", true);
				connStrField.set("value", this._buildSampleOracleConnectString("", ""));
				connStrEdit.set("disabled", false);
				connStrReset.set("disabled", true);
				this._isConnStrEdited = false;
			}
			else if (selType == "teradata") {
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_dbname_label;
				domStyle.set(portField.domNode, "display", "none");
				portField.set("disabled", true);  // Prevent validation
				domStyle.set(dbNameField.domNode, "display", "");
				driverField.set("value", SharedProperties.jdbcConfiguration_teradatadriver);
				driverField.set("disabled", true);
				hostField.set("disabled", false);
				hostField.set("value", "");
				domStyle.set(hostAsterisk, "visibility", "visible");
				dbNameField.set("disabled", false);
				dbNameField.set("value", "");
				domStyle.set(portAsterisk, "visibility", "visible");
				domStyle.set(connMgrEntry, "display", "block");
				domStyle.set(connMgrAsterisk, "visibility", "hidden");
				connMgrField.set("disabled", false);
				connMgrField.set("required", false);
				connMgrField.set("value", "");
				connStrField.set("disabled", true);
				connStrField.set("value", this._buildSampleTeradataConnectString("", ""));
				connStrEdit.set("disabled", false);
				connStrReset.set("disabled", true);
				this._isConnStrEdited = false;
			}
			else if (selType == "federation") {
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_port_label;
				domStyle.set(portField.domNode, "display", "");
				domStyle.set(dbNameField.domNode, "display", "none");
				dbNameField.set("disabled", true);  // Prevent validation
				driverField.set("value", SharedProperties.jdbcConfiguration_federationdriver);
				driverField.set("disabled", true);
				hostField.set("disabled", false);
				hostField.set("value", "");
				domStyle.set(hostAsterisk, "visibility", "visible");
				portField.set("disabled", false);
				portField.set("value", "");
				domStyle.set(portAsterisk, "visibility", "visible");
				dsnField.set("disabled", false);
				dsnField.set("value",  "");
				domStyle.set(dsnEntry, "display", "block");
				domStyle.set(dsnAsterisk, "visibility", "visible");
				connStrField.set("disabled", true);
				connStrField.set("value", this._buildSampleFederationConnectString("", ""));
				connStrEdit.set("disabled", false);
				connStrReset.set("disabled", true);
				this._isConnStrEdited = false;
			}
			else {
				portLabel.innerHTML = ResourceBundle.databaseConfigurationDialog_port_label;
				domStyle.set(portField.domNode, "display", "");
				domStyle.set(dbNameField.domNode, "display", "none");
				dbNameField.set("disabled", true);  // Prevent validation
				driverField.set("value", "");
				driverField.set("disabled", false);
				hostField.set("value", "");
				hostField.set("disabled", true);
				domStyle.set(hostAsterisk, "visibility", "hidden");
				portField.set("disabled", true);
				portField.set("value", "");
				domStyle.set(portAsterisk, "visibility", "hidden");
				domStyle.set(connMgrEntry, "display", "block");
				domStyle.set(connMgrAsterisk, "visibility", "hidden");
				connMgrField.set("disabled", false);
				connMgrField.set("required", false);
				connMgrField.set("value", "");
				connStrField.set("disabled", false);
				connStrField.set("value", "");
				connStrEdit.set("disabled", true);
				connStrReset.set("disabled", true);
				this._isConnStrEdited = true;
			}
			
			// S1116302 - Wipe out user ID and password
			userField.set("value", "");
			passwordField.set("value", "");
			
			this._lastTypeSelection = selType;
			this._ignoreChange = false;
		},
		
		_onEditClick:function(evt) {
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var hostAsterisk = dom.byId("DatabaseConfigurationDialogHostAsterisk" + this._getIDSuffix());
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var portAsterisk = dom.byId("DatabaseConfigurationDialogPortAsterisk" + this._getIDSuffix());
			var dbNameField = this.findChildByID("DatabaseConfigurationDialogDBName");
			var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
			var sidAsterisk = dom.byId("DatabaseConfigurationDialogSIDAsterisk" + this._getIDSuffix());
			var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
			var dsnAsterisk = dom.byId("DatabaseConfigurationDialogDsnAsterisk" + this._getIDSuffix());
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var connStrEdit = this.findChildByID("DatabaseConfigurationDialogConnectStringEdit");
			var connStrReset = this.findChildByID("DatabaseConfigurationDialogConnectStringReset");

			this._ignoreChange = true;
			
			hostField.set("value", "");
			hostField.set("disabled", true);
			if (this._lastTypeSelection == "oracle") {
				portField.set("value", "");
				portField.set("disabled", true);
				domStyle.set(sidAsterisk, "visibility", "hidden");
				sidField.set("value", "");
				sidField.set("disabled", true);
			}
			else if (this._lastTypeSelection == "federation"){
				portField.set("value", "");
				portField.set("disabled", true);
				domStyle.set(dsnAsterisk, "visibility", "hidden");
				dsnField.set("value", "");
				dsnField.set("disabled", true);
			}
			else {  // Teradata
				dbNameField.set("value", "");
				dbNameField.set("disabled", true);
			}
			domStyle.set(hostAsterisk, "visibility", "hidden");
			domStyle.set(portAsterisk, "visibility", "hidden");
			connStrField.set("disabled", false);
			this._focusIfNotNull(connStrField);
			connStrEdit.set("disabled", true);
			connStrReset.set("disabled", false);
			this._isConnStrEdited = true;
			
			this._ignoreChange = false;
		},
		
		_onResetClick:function(evt) {
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var hostAsterisk = dom.byId("DatabaseConfigurationDialogHostAsterisk" + this._getIDSuffix());
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var portAsterisk = dom.byId("DatabaseConfigurationDialogPortAsterisk" + this._getIDSuffix());
			var dbNameField = this.findChildByID("DatabaseConfigurationDialogDBName");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var connStrEdit = this.findChildByID("DatabaseConfigurationDialogConnectStringEdit");
			var connStrReset = this.findChildByID("DatabaseConfigurationDialogConnectStringReset");
			var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
			var sidAsterisk = dom.byId("DatabaseConfigurationDialogSIDAsterisk" + this._getIDSuffix());
			var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
			var dsnAsterisk = dom.byId("DatabaseConfigurationDialogDsnAsterisk" + this._getIDSuffix());
			
			this._ignoreChange = true;
			
			hostField.set("disabled", false);
			domStyle.set(hostAsterisk, "visibility", "visible");
			domStyle.set(portAsterisk, "visibility", "visible");
			domStyle.set(sidAsterisk, "visibility", "visible");
			domStyle.set(dsnAsterisk, "visibility", "visible");
			this._focusIfNotNull(hostField);
			if (this._lastTypeSelection == "oracle") {
				portField.set("disabled", false);
				sidField.set("disabled", false);
				connStrField.set("value", this._buildSampleOracleConnectString("", ""));
			}
			else if (this._lastTypeSelection == "federation") {
				portField.set("disabled", false);
				dsnField.set("disabled", false);
				connStrField.set("value", this._buildSampleFederationConnectString("", ""));
			}
			else {  // Teradata
				dbNameField.set("disabled", false);
				connStrField.set("value", this._buildSampleTeradataConnectString("", ""));
			}
			connStrField.set("disabled", true);
			connStrEdit.set("disabled", false);
			connStrReset.set("disabled", true);
			this._isConnStrEdited = false;
			
			this._ignoreChange = false;
		},
		
		_onHostChange:function(evt) {
			if (this._ignoreChange == true) {
				return;
			}
			
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var dbNameField = this.findChildByID("DatabaseConfigurationDialogDBName");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			
			var host = hostField.get("value");
			if (host != null) {
				host = host.trim();
			}
			if (this._lastTypeSelection == "oracle") {
				var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
				connStrField.set("value", this._buildSampleOracleConnectString(host, portField.get("value"), sidField.get("value")));
			}
			else if (this._lastTypeSelection == "federation") {
				var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
				connStrField.set("value", this._buildSampleFederationConnectString(host, portField.get("value"), dsnField.get("value")));
			}
			else if(this._lastTypeSelection == "teradata"){
				connStrField.set("value", this._buildSampleTeradataConnectString(host, dbNameField.get("value")));
			}
			else {  
				connStrFiedl.set("value", "");
			}
		},
		
		_onPortChange:function(evt) {
			if (this._ignoreChange == true) {
				return;
			}
			
			if (this._lastTypeSelection != "oracle" && this._lastTypeSelection != "federation") {
				return;
			}
			
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
			var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
			
			var host = hostField.get("value");
			if (host != null) {
				host = host.trim();
			}
			if (this._lastTypeSelection == "oracle") {
				connStrField.set("value", this._buildSampleOracleConnectString(host, portField.get("value"), sidField.get("value")));
			}
			else if (this._lastTypeSelection == "federation") {
				connStrField.set("value", this._buildSampleFederationConnectString(host, portField.get("value"), dsnField.get("value")));
			}
		},
		
		_onDBNameChange:function(evt) {
			if (this._ignoreChange == true) {
				return;
			}
			
			if (this._lastTypeSelection == "oracle" || this._lastTypeSelection == "federation") {
				return;
			}
			
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var dbNameField = this.findChildByID("DatabaseConfigurationDialogDBName");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			
			var host = hostField.get("value");
			if (host != null) {
				host = host.trim();
			}
			connStrField.set("value", this._buildSampleTeradataConnectString(host, dbNameField.get("value")));
		},
		
		_onSidFieldChange:function(evt){
			if (this._ignoreChange == true) {
				return;
			}
			
			if (this._lastTypeSelection != "oracle") {
				return;
			}
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
			
			var host = hostField.get("value");
			if (host != null) {
				host = host.trim();
			}
			
			connStrField.set("value", this._buildSampleOracleConnectString(host, portField.get("value"), sidField.get("value")));
		},
		
		_onDsnFieldChange:function(evt){
			if (this._ignoreChange == true) {
				return;
			}
			
			if (this._lastTypeSelection != "federation") {
				return;
			}
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
			
			var host = hostField.get("value");
			if (host != null) {
				host = host.trim();
			}
			
			connStrField.set("value", this._buildSampleFederationConnectString(host, portField.get("value"), dsnField.get("value")));
		},
		
		_onDriverChange:function(evt) {
			this._updateTestConnection();
		},
		
		_onConnStrChange:function(evt) {
			this._updateTestConnection();
		},
		
		_updateTestConnection:function() {
			var driverField = this.findChildByID("DatabaseConfigurationDialogDriver");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var testConnection = this.findChildByID("DatabaseConfigurationDialogTestConnection");
			
			var driver = driverField.get("value");
			var connStr = connStrField.get("value");
			if (driver == null || driver.length == 0 || connStr == null || connStr.length == 0) {
				testConnection.set("disabled", true);
			}
			else {
				testConnection.set("disabled", false);
			}
		},
		
		_buildSampleOracleConnectString:function(host/*String*/, port/*Integer*/, serviceName /*String*/) {
			// Format = jdbc:oracle:thin:@<host>:<port>/<service_name>
			var connStr = "jdbc:oracle:thin:@";
			if (StringUtils.isEmpty(host) != true) {
				connStr += host;
			}else{
				connStr +="<host>";
			}
			connStr += ":";
			// isNaN("") == false, so we need to check for type string
			if (port != null && typeof port != "string" && isNaN(port) == false) {
				connStr += port;
			}else{
				connStr += "<port>";
			}
			
			connStr += "/";
			if(StringUtils.isEmpty(serviceName) == false){
				connStr += serviceName;
			}else{
				connStr += "<service_name>";
			}
			
			return connStr;
		},
		
		_buildSampleFederationConnectString:function(host/*String*/, port/*Integer*/, dsn /*String*/) {
			// Format = jdbc:sastkts://<host>:<port>?constring=(DSN=<dsn>)
			var connStr = "jdbc:sastkts://";
			if (StringUtils.isEmpty(host) != true) {
				connStr += host;
			}else{
				connStr +="<host>";
			}
			connStr += ":";
			// isNaN("") == false, so we need to check for type string
			if (port != null && typeof port != "string" && isNaN(port) == false) {
				connStr += port;
			}else{
				connStr += "<port>";
			}
			
			connStr += "?constring=(DSN=";
			if(StringUtils.isEmpty(dsn) == false){
				connStr += dsn;
			}else{
				connStr += "<dsn>";
			}
			connStr += ")";
				
			return connStr;
		},
		
		_buildSampleTeradataConnectString:function(host/*String*/, db/*String*/) {
			// Format = jdbc:teradata://<host>/Database=<db>
			var connStr = "jdbc:teradata://";
			if (StringUtils.isEmpty(host) != true) {
				connStr += host;
			}else{
				connStr += "<host>"; 
			}
			connStr += "/Database=";
			if (StringUtils.isEmpty(db) != true) {
				connStr += db;
			}else{
				connStr += "<database>";
			}
			return connStr;
		},
		
		_focusIfNotNull:function(input){
			if (input != null) {
				focus.focus(input);
			}
		},
		
		//set this to get notified when the dialog is finished
		okHandler:null,
		
		_onErrorDetailsChange:function(value/*Error*/) {
			var pm = this._getPresentationModelInstance();
		    if(pm == null){
		        return;
		    }
			if (value != null) {
				if(this._errorDetailsDialog == null){
			        this._errorDetailsDialog = new TextFieldDialog();
				}
				this._errorDetailsDialog.set("title", pm.get("errorTitle"));
				this._errorDetailsDialog.set("textContent", value);
				this._errorDetailsDialog.show();
			}
			else {
				if (this._errorDetailsDialog != null) {
					this._errorDetailsDialog.hide();
				}
			}
		},
		
		show:function(container/*UI Container Object*/) {
			this._showContainer = container;
			this._initialShow = true;
			if (this.__downloadComplete === false) {
				this._showAfterDownload = true;
			}
			else {
				this._internalShow();
			}
		},
		
		_internalShow:function() {
			var i;
			var host, connStr;
			var self = this;
			var pm = this._getPresentationModelInstance();
			var dialog = this.findChildByID("DatabaseConfigurationDialog");
			var dialogSubmit = this.findChildByID("DatabaseConfigurationDialogSubmit");
			var nameField = this.findChildByID("DatabaseConfigurationDialogName");
			var descriptionField = this.findChildByID("DatabaseConfigurationDialogDescription");
			var typeSelector = this.findChildByID("DatabaseConfigurationDialogType");
			var driverField = this.findChildByID("DatabaseConfigurationDialogDriver");
			var hostField = this.findChildByID("DatabaseConfigurationDialogHost");
			var portField = this.findChildByID("DatabaseConfigurationDialogPort");
			var dbNameField = this.findChildByID("DatabaseConfigurationDialogDBName");
			var connStrField = this.findChildByID("DatabaseConfigurationDialogConnectString");
			var connStrEdit = this.findChildByID("DatabaseConfigurationDialogConnectStringEdit");
			var connStrReset = this.findChildByID("DatabaseConfigurationDialogConnectStringReset");
			var connStrHelp = this.findChildByID("DatabaseConfigurationDialogConnectStringHelp");
			var userField = this.findChildByID("DatabaseConfigurationDialogUserID");
			var passwordField = this.findChildByID("DatabaseConfigurationDialogPassword");
			var sidField = this.findChildByID("DatabaseConfigurationDialogSID");
			var dsnField = this.findChildByID("DatabaseConfigurationDialogDsn");
			var connMgrField = this.findChildByID("DatabaseConfigurationDialogConnMgr");
			var testConnection = this.findChildByID("DatabaseConfigurationDialogTestConnection");
			var deleteCredentials = this.findChildByID("ConfigurationDialogDeleteCredentials");
			
			if (dialog != null) {
				on(dialog, "hide", function() {
					if (self._showContainer != null) {
						self._showContainer.removeChild(self);
						self.destroy();
					}
				});
				
				on(dialog, "show", function() {
					if (self._initialShow === true) {
						self._initialShow = false;
						
						on(typeSelector, "change", _lang.hitch(self, self._onTypeChange));
						on(driverField, "change", _lang.hitch(self, self._onDriverChange));
						on(hostField, "change", _lang.hitch(self, self._onHostChange));
						on(portField, "change", _lang.hitch(self, self._onPortChange));
						on(connStrField, "change", _lang.hitch(self, self._onConnStrChange));
						on(dbNameField, "change", _lang.hitch(self, self._onDBNameChange));
						on(sidField, "change", _lang.hitch(self, self._onSidFieldChange));
						on(dsnField, "change", _lang.hitch(self, self._onDsnFieldChange));
						on(connStrEdit, "click", _lang.hitch(self, self._onEditClick));
						on(connStrReset, "click", _lang.hitch(self, self._onResetClick));
						connStrHelp.set("message", ResourceBundle.databaseConfigurationDialog_connectstring_help);
					}
				});
				
				if (dialogSubmit != null) {
					on(dialogSubmit, "click", function(evt) {
						evt.preventDefault();
						dialog.validate();
						if (dialog.isValid()) {
							host = hostField.get("value");
							if (host != null) {
								host = host.trim();
							}
							connStr = connStrField.get("value");
							if (connStr != null) {
								connStr = connStr.trim();
							}
							var server = self._createServerInstance(self._lastTypeSelection);  // Create a new JdbcServer model object to pass to the PM with all the values
							server.set("name", nameField.get("value"));
							server.set("description", descriptionField.get("value"));
							server.set("hostName", "");  // Mid-tier won't accept a null host name (violates a null constraint in the SRVRRCS table)
							if (self._lastTypeSelection == "oracle") {
								server.set("driverClassName", SharedProperties.jdbcConfiguration_oracledriver);
								if (self._isConnStrEdited == true) {
									server.set("connectionString", connStr);
								}
								else {
									server.set("hostName", host);
									server.set("port", portField.get("value"));
									server.get("properties").service_name = sidField.get("value");
								}
							}
							else if (self._lastTypeSelection == "teradata") {
								server.set("driverClassName", SharedProperties.jdbcConfiguration_teradatadriver);
								server.set("connectionManager", connMgrField.get("value"));
								if (self._isConnStrEdited == true) {
									server.set("connectionString", connStr);
								}
								else {
									server.set("hostName", host);
									server.set("databaseName", dbNameField.get("value"));
								}
							}
							else if (self._lastTypeSelection == "federation") {
								server.set("driverClassName", SharedProperties.jdbcConfiguration_federationdriver);
								if (self._isConnStrEdited == true) {
									server.set("connectionString", connStr);
								}
								else {
									server.set("hostName", host);
									server.set("port", portField.get("value"));
									server.get("properties").dsn = dsnField.get("value");
								}
							}
							else {
								server.set("driverClassName", driverField.get("value"));
								server.set("connectionManager", connMgrField.get("value"));
								server.set("connectionString", connStr);
							}
							
							pm.set("userName", userField.get("value"));
							pm.set("password", passwordField.get("value"));
							
							pm.handleSubmit(server,
								function(result) {
									console.log(result);
									if (self.okHandler != null) {
										self.okHandler(self.getJdbcServer());
									}
									dialog.hide();
								},
								function(error) {
									console.error(error);
								}
							);
						}
					});
				}
				
				if (deleteCredentials != null)
				{
					on(deleteCredentials, "click", _lang.hitch(this, function(evt) 
					{
						var confirmRemoveDialog = new ConfirmationDialog({
							title: ResourceBundle.credentialsRemove_title,
							content: ResourceBundle.credentialsRemove_content,
							closable: false
						});
						
						confirmRemoveDialog.set("submitButtonHandler", _lang.hitch(this, function(){
							var server = pm.get("jdbcServer");
							if (server != null)
							{
								pm.deleteCredentials();
							}				
							confirmRemoveDialog.destroyRecursive();
						 }));
						
						confirmRemoveDialog.set("cancelButtonHandler", _lang.hitch(this, function(){
							confirmRemoveDialog.destroyRecursive();
						 }));
						
						confirmRemoveDialog.show();
					}));
				}
				
				if (testConnection != null) {
					on(testConnection, "click", function() {
						dialog.validate();
						if (dialog.isValid()) {
							var user = userField.get("value");
							var password = passwordField.get("value");
							if (password == null) {
								password = "";
							}
							
							// Convert password to base64
							var pwBytes = [];
							for (var j = 0; j < password.length; j++) {
								pwBytes.push(password.charCodeAt(j));
							}
							var password_base64 = window.btoa(pwBytes);
							
							connStr = connStrField.get("value");
							if (connStr != null) {
								connStr = connStr.trim();
							}
							
							var server = pm.get("jdbcServer");
							var domain = server!=null ? server.get("authDomain") : "";
							self._controller.testJDBCConnection(driverField.get("value"), connStr, user, password_base64, domain, null,
								_lang.hitch(self, self._testSuccessHandler), _lang.hitch(self, self._testErrorHandler));
						}
					});
				}
				
				dialog.show();
			}
		},
		
		_saveSuccessHandler:function(result) {
			this._pendingSaveCalls--;
			if (this._pendingSaveCalls === 0 && this._dialogHasErrors === false) {
				var dialog = this.findChildByID("DatabaseConfigurationDialog");
				dialog.hide();
			}
		},
		
		_saveErrorHandler:function(error) {
			this._pendingSaveCalls--;
			this._dialogHasErrors = true;
			if (this._pendingSaveCalls === 0) {
				var msgbox = new MessageBox({title:ResourceBundle.databaseConfigurationDialog_title, showOkButtonOnly:true});
				msgbox.set("message", error.get("message"));
				msgbox.show();
			}
		},
		
		_testSuccessHandler:function(result) {
			var msgbox = new MessageBox({title:ResourceBundle.databaseConfigurationDialog_testconnection_msgbox_title, showOkButtonOnly:true});
			msgbox.set("message", ResourceBundle.databaseConfigurationDialog_testconnection_msgbox_success);
			msgbox.show();
		},
		
		_testErrorHandler:function(error) {
			var resolvedMessage = StringUtils.resolveMessage(error.get("message"), ResourceBundle);
			if (resolvedMessage == null) {
				resolvedMessage = error.get("message");
			}
			var msgbox = new MessageBox({title:ResourceBundle.databaseConfigurationDialog_testconnection_msgbox_title, showOkButtonOnly:true});
			msgbox.set("message", resolvedMessage);
			msgbox.show();
		},
		
		// ** NOTE: This function was copied from _DataDirectorUIBase, because we haven't been able to figure
		// out a way to get it into a mixin so that it's available to multiple classes. **
		// override - this is a combination of the set method in Stateful and the set method in _WidgetBase
		// which allows outside classes to bind to properties this widget exposes through getters and setters
		// This essentially makes the widget act like a true Stateful when a property changes by using its setter
		set:function(name, value){
			if(typeof name === "object"){
				for(var x in name){
					if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
						this.set(x, name[x]);
					}
				}
				return this;
			}
			var names = this._getAttrNames(name),
				oldValue = this._get(name, names),
				setter = this[names.s],
				result;
			if(typeof setter === "function"){
				// use the explicit setter
				result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
			}else{
				// no setter so set attribute directly
				return this.inherited(arguments);
			}
			if(this._watchCallbacks){
				var self = this;
				// If setter returned a promise, wait for it to complete, otherwise call watches immediately
				when(result, function(){
					self._watchCallbacks(name, oldValue, self.get(name));
				});
			}
			return this; 
		},
		
		_unbindPresentationModel:function() {
			BindingUtils.Unbind(this._jdbcServerWatcher);
			BindingUtils.Unbind(this._userNameWatcher);
			BindingUtils.Unbind(this._passwordWatcher);
			BindingUtils.Unbind(this._errorDetailsWatcher);
		},
		
		_createServerInstance:function(type /*String - value of _lastTypeSelection*/){
			switch(type){
				case "oracle":
					return new OracleJdbcServer();
				case "federation":
					return new FederationJdbcServer();
				case "teradata":
					return new TeradataJdbcServer();
			}
			return new JdbcServer();
		},
		
		destroy:function() {
			this._unbindPresentationModel();
			var pm = this._getPresentationModelInstance();
			if (pm != null && _lang.isFunction(pm.destroy)) {
				pm.destroy();
			}
			if(this._errorDetailsDialog != null){
				this._errorDetailsDialog.destroyRecursive();
			}
		},
		
		constructor:function(params, srcNodeRef) {
			var p = params || {};
			if(p.href == null) {
				p.href = "script/js/com/sas/dm/datadirector/views/markup/DatabaseConfigurationDialog.html";
			}
			//need to check for the controller
			if (p.controller != null) {
				this._controller = p.controller;
			}
			// Manually call the super class constructor
			this.inherited(arguments,[p, srcNodeRef]);
		}
	});
	
	return DatabaseConfigurationDialog;
});
