define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/config",
        "dojo/query",
        "dojo/dom",
        "dojo/dom-class",
        "dojo/dom-construct",
        "dojo/dom-style",
        "dojo/Evented",
        "dojo/on",
        "dojo/when",
        "dijit/registry",
        "dijit/focus",
        "dijit/Tooltip",
        "com/sas/dm/datadirector/widgets/MessageBox",
        "com/sas/dm/datadirector/views/components/ModelSelectViewDialog",
        "com/sas/dm/datadirector/views/presentation/HadoopConfigPanePM",
        "com/sas/dm/common/components/TextFieldDialog",
        "com/sas/dm/common/components/ConfirmationDialog",
        "com/sas/dm/common/util/BindingUtils",
        "com/sas/dm/common/components/base/util/ContextUtils",
        "com/sas/dm/common/util/UIUtils",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/datadirector/mixins/_ConfigDialogMixin",
        "com/sas/dm/datadirector/views/components/_IsBusyUIBase",
        "com/sas/dm/datadirector/mixins/_DataLoaderFormMixin",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector",
        "com/sas/dm/datadirector/util/SharedProperties",
		"com/sas/dm/datadirector/util/ResourcePathBuilder"],
        function(declare, _lang, config, query, dom, domClass, domConstruct, domStyle, Evented, on, when, registry, focus, Tooltip, MessageBox, ModelSelectViewDialog, HadoopConfigPanePM, TextFieldDialog, ConfirmationDialog, BindingUtils, ContextUtils, UIUtils, StringUtils, _ConfigDialogMixin, _IsBusyUIBase, _DataLoaderFormMixin, ResourceBundle, SharedProperties, ResourcePathBuilder) {
	var HadoopConfigPane = declare([_IsBusyUIBase, _ConfigDialogMixin, _DataLoaderFormMixin], {
		_pm:null,
		_pmBound:false,
		_controller:null,
		_initializeAfterDownload:false,
		_hiveServerWatcher:null,
		_hiveHostNameWatcher:null,
		_hivePortWatcher:null,
		_impalaHostNameWatcher:null,
		_impalaPortWatcher:null,
		_userIdWatcher:null,
		_userPasswordWatcher:null,
		_errorDetailsWatcher:null,
		_initialShow:false,
		_oozieWatcher:null,
		_preferredSQLEnvironmentWatcher:null,
		_preferredSQLEnvActionWatcher:null,
		_busyWatcher: null,
		_userIdHelpWatcher:null,
		_preferredRuntimeEnvironmentWatcher:null,
		_preferredRuntimeEnvActionWatcher:null,
		
		_hdfsHomePath:null,
		_setHdfsHomePathAttr:function(value){
			this._hdfsHomePath=value;
		},
		_getHdfsHomePathAttr:function(){
			return this._hdfsHomePath;
		},
		
		_setContent:function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
			
			cont = cont.replace(/([\w\d_]+)_\$I18N\$/g,function(match, p1){
				console.debug(p1);
				return ResourceBundle[p1];
			});
			return this.inherited(arguments, [cont, isFakeContent]);
		},

		/*Override*/
		__onDownloadComplete:function() {
			this.inherited(arguments);
			
			if (this._initializeAfterDownload) {
				this._initializeAfterDownload = false;
				this._internalInitialize();
			}
		},

		/*Override*/
		// if pmClass were passed in as a parameter to the constructor, we would not have
		// to override _getPresentationModelInstance, but since we're supplying the implementation
		// directly, this is the place to do it.
		_getPresentationModelInstance:function() {
			if (this._pm == null){
				// this class doesn't need to be context aware, but it does need a reference
				// to the most local context to pass to the PM, so we use ContextUtils.FindContext
				var context = ContextUtils.FindContext(this);
				this._pm = new HadoopConfigPanePM({"context":context});
			}
			return this._pm;
		},
		
		/*Override*/
		// this is where we're actually binding any properties exposed in the PM to the view
		_bindPresentationModel:function() {
			var pm = this._getPresentationModelInstance();
			if (pm == null){
				console.error("No PM defined in HadoopConfigPane.js");
				return;
			}

			var hiveHelp = this.findChildByID("ConfigurationDialogHiveHelp");
			if (hiveHelp != null) {
				hiveHelp.set("message", ResourceBundle.hadoopConfigPane_hiveServer_helpText);
			}
			var impalaHelp = this.findChildByID("ConfigurationDialogImpalaHelp");
			if (impalaHelp != null) {
				impalaHelp.set("message", ResourceBundle.hadoopConfigPane_impalaServer_helpText);
			}
			var sqlHelp = this.findChildByID("ConfigurationDialogSqlEnvironmentHelp");
			if (sqlHelp != null) {
				sqlHelp.set("message", ResourceBundle.hadoopConfigPane_sqlEnvironment_helpText);
			}
			var runtimeHelp = this.findChildByID("ConfigurationDialogPreferredRuntimeHelp");
			if (runtimeHelp != null) {
				runtimeHelp.set("message", ResourceBundle.hadoopConfigPane_preferredRuntime_helpText);
			}
			var oozieHelp = this.findChildByID("ConfigurationDialogOozieHelp");
			if (oozieHelp != null) {
				oozieHelp.set("message", ResourceBundle.hadoopConfigPane_oozie_helpText);
			}
            var sasworkspaceHelp = this.findChildByID("ConfigurationDialogWorkspaceServerHelp");
            if (sasworkspaceHelp != null) {
                sasworkspaceHelp.set("message", ResourceBundle.hadoopConfigPane_sasWorkspaceServer_helpText);
            }
			
			this._busyWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onBusyChange), pm, "isBusy"); // call before setting controller
			
			pm.set("controller", this._controller);
			this._hiveServerWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onHiveServerChange), pm, "hiveServer");
			this._impalaServerWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onImpalaServerChange), pm, "impalaServer");
			// this._userIdWatcher  = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onUserIdChange), pm, "userId");
			// this._userPasswordWatcher  = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onUserPasswordChange), pm, "password");
			this._errorDetailsWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onErrorDetailsChange), pm, "errorDetails");
			this._hiveHostNameWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onHiveHostNameChange), pm, "hiveHostName");
			this._hivePortWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onHivePortChange), pm, "hivePort");
            this._impalaHostNameWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onImpalaHostNameChange), pm, "impalaHostName");
            this._impalaPortWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onImpalaPortChange), pm, "impalaPort");			
			this._oozieWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onOozieChange), pm, "oozieUrl");
			this._preferredSQLEnvironmentWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onPreferredSQLEnvironmentChange), pm, "preferredSQLEnvironment");
			this._preferredSQLEnvActionWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onPreferredSQLEnvironmentActionChange), pm, "preferredSQLEnvironmentAction");
			//this._userIdHelpWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onUserIdHelpChange), pm, "userIdHelp");
			this._preferredRuntimeEnvironmentWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onPreferredRuntimeEnvironmentChange), pm, "preferredRuntimeEnvironment");
			this._preferredRuntimeEnvActionWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onPreferredRuntimeEnvironmentActionChange), pm, "preferredRuntimeEnvironmentAction");
            this._preferredSASWorkspaceServerWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onPreferredSASWorkspaceServerChange), pm, "preferredSASWorkspaceServer");
            this._preferredSASWorkspaceServerActionWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onPreferredSASWorkspaceServerActionChange), pm, "preferredSASWorkspaceServerAction");

            this._pmBound = true;
			
			if(this.get("loadInventoryInfo")) {
			    pm._updateInventoryInfo();    
			}			
		},
		
		_onBusyChange: function(value){
			this.set("isBusy", value);
		},
		
		_updateHiveServerConfigured:function(){
			var hiveHostField = this.findChildByID("ConfigurationDialogHiveHost");
			var hivePortField = this.findChildByID("ConfigurationDialogHivePort");
			this._updateHiveServerConfiguredEx(hiveHostField, hivePortField);
		},

		// TODO: this validation should be in the PM
		_updateHiveServerConfiguredEx:function(hiveHostField, hivePortField){
			var hiveServerConfigured = false;
			if(hiveHostField != null && hivePortField != null)
			{
				hiveServerConfigured = StringUtils.isEmpty(hiveHostField.get("value")) == false && StringUtils.isEmpty(hivePortField.get("value")) == false;
			}
			
			if(this._controller != null)
				this._controller.set("hiveServerConfigured", hiveServerConfigured);	
		},
		
        _onPreferredRuntimeEnvironmentActionChange: function(value /*object*/){
            //string token passed into .findChildById maps to id in html (GeneralConfigPane.html)
            var preferredRuntimeEnvWidget = this.findChildByID("HadoopConfigurationPreferredRuntime");
            if(preferredRuntimeEnvWidget != null && value != null) {
                preferredRuntimeEnvWidget.set("action", value);
            }           
        },
        
        _onHiveHostNameChange:function(value/*String*/) {
            var hiveHostField = this.findChildByID("ConfigurationDialogHiveHost");
            if (hiveHostField != null && value != null) {
                hiveHostField.set("value", value);               
            }            
        },

        _onHivePortChange:function(value/*String*/) {
            var hivePortField = this.findChildByID("ConfigurationDialogHivePort");
            if (hivePortField != null && value != null) {
                hivePortField.set("value", value);               
            }           
        },
        
		_onHiveServerChange:function(value/*HiveServer*/) {
			var hiveHostField = this.findChildByID("ConfigurationDialogHiveHost");
			if (hiveHostField != null) {
				hiveHostField.set("value", value != null ? value.get("hostName") : null);				
			}
			
			var hivePortField = this.findChildByID("ConfigurationDialogHivePort");
			if (hivePortField != null) {
				hivePortField.set("value", value != null ? value.get("port") : null);				
			}			
			this._updateHiveServerConfiguredEx(hiveHostField, hivePortField);
			
			hiveHostField = null;	
		},
		
        _onImpalaHostNameChange:function(value/*String*/) {
            var impalaHostField = this.findChildByID("ConfigurationDialogImpalaHost");
            if (impalaHostField != null && value != null) {
                impalaHostField.set("value", value);             
            }         
        },

        _onImpalaPortChange:function(value/*String*/) {
            var impalaPortField = this.findChildByID("ConfigurationDialogImpalaPort");
            if (impalaPortField != null && value != null) {
                impalaPortField.set("value", value);               
            }            
        },
		
		_onImpalaServerChange:function(value/*ImpalaServer*/) {
			var impalaHostField = this.findChildByID("ConfigurationDialogImpalaHost");
			if (impalaHostField != null) {
				impalaHostField.set("value", value != null ? value.get("hostName") : null);				
			}
			
			var impalaPortField = this.findChildByID("ConfigurationDialogImpalaPort");
			if (impalaPortField != null) {
				var impalaPort = null;
				if(value){
					impalaPort = value.get("port");
					if(impalaPort < 0)
						impalaPort = null;
				}
				impalaPortField.set("value", impalaPort);				
			}
		},
		
		_onOozieChange:function(value/*String*/) {
			var oozie = this.findChildByID("ConfigurationDialogOozie");
			oozie.set("value", value);
		},
		
        _onPreferredSQLEnvironmentActionChange: function(value /*object*/){
            //string token passed into .findChildById maps to id in html (HadoopConfigPane.html)
            var preferredSQLEnvWidget = this.findChildByID("ConfigurationDialogPreferredSQLEnv");
	        if(preferredSQLEnvWidget != null && value != null) {
	            preferredSQLEnvWidget.set("action", value);
	        }           
	    },
		
	    // dawong - 27 Jul 2015
	    // S1192570 - The Impala host/port fields should be required when the
	    // preferred SQL environment is Impala.  (The fields for the Hive
	    // host/port remain required all the time.)
		_onPreferredSQLEnvironmentChange: function(value /*String - SharedProperties.preferredSqlEnvironment_xxx*/){
		    var isImpala = (value === SharedProperties.preferredSqlEnvironment_impala);
		    
            var impalaHostInput = this.findChildByID("ConfigurationDialogImpalaHost");
	        var impalaPortInput = this.findChildByID("ConfigurationDialogImpalaPort");
	        var impalaSelect = this.findChildByID("ConfigurationDialogPreferredSQLEnv");
	        
	        // var impalaHostAsterisk = dom.byId("ImpalaHostAsterisk" + this._getIDSuffix());
	        // var impalaPortAsterisk = dom.byId("ImpalaPortAsterisk" + this._getIDSuffix());
	        //
	        // impalaHostInput.required = isImpala;
	        // impalaPortInput.required = isImpala;
	        //
	        // if (isImpala) {
	        //     domClass.remove(impalaHostAsterisk, "displayNone");
	        //     domClass.remove(impalaPortAsterisk, "displayNone");
	        // }
	        // else {
	        //     domClass.add(impalaHostAsterisk, "displayNone");
	        //     domClass.add(impalaPortAsterisk, "displayNone");
	        // }
	        
	        impalaSelect.set("value", value);

	        // cycle focus through the impala host/port fields to refresh
	        // their validation indicators
            this._focusIfNotNull(impalaHostInput);
            this._focusIfNotNull(impalaPortInput);
            // this is causing a stackoverflow exception in chrome when the value in the select box is changed
            //this._focusIfNotNull(impalaSelect);
		},

        _onPreferredSASWorkspaceServerActionChange: function(value /*object*/){
            //string token passed into .findChildById maps to id in html (HadoopConfigPane.html)
            var preferredWorkspaceWidget = this.findChildByID("ConfigurationDialogSASWorkspaceServerSelect");
            if(preferredWorkspaceWidget != null && value != null) {
                preferredWorkspaceWidget.set("action", value);
            }
        },

        _onPreferredSASWorkspaceServerChange: function(value /*String*/){
            var wsServerSelect = this.findChildByID("ConfigurationDialogSASWorkspaceServerSelect");
            wsServerSelect.set("value", value);
        },

        _onPreferredRuntimeEnvironmentChange: function(value /*String - SharedProperties.executionEnvironmentInternalName_xxx*/){
	        var runtimeSelect = this.findChildByID("HadoopConfigurationPreferredRuntime");

	        runtimeSelect.set("value", value);
	    },

		_onErrorDetailsChange:function(value/*Error*/) {
			var pm = this._getPresentationModelInstance();
		    if(pm == null){
		        return;
		    }
			if (value != null) {
				if(this._errorDetailsDialog == null){
		        	this._errorDetailsDialog = new TextFieldDialog();
				}
				this._errorDetailsDialog.set("title", pm.get("errorTitle"));
				this._errorDetailsDialog.set("textContent", value);
				this._errorDetailsDialog.show();
			}
			else {
				if (this._errorDetailsDialog != null) {
					this._errorDetailsDialog.hide();
				}
			}
		},
		
		_focusIfNotNull:function(input){
			if (input != null) {
				focus.focus(input);
			}
		},
		
		_trimIfNotNull:function(value) {
			if (value == null) {
				return null;
			}
			return value.trim();
		},
		
		_internalInitialize:function() {
			this.connectChildren(false);
			
			var self = this;
			var pm = this._getPresentationModelInstance();
			var dialogHiveHost = this.findChildByID("ConfigurationDialogHiveHost");
			var dialogHivePort = this.findChildByID("ConfigurationDialogHivePort");
			var dialogImpalaHost = this.findChildByID("ConfigurationDialogImpalaHost");
			var dialogImpalaPort = this.findChildByID("ConfigurationDialogImpalaPort");
			var dialogOozie = this.findChildByID("ConfigurationDialogOozie");
			var preferredSQLEnvironment = this.findChildByID("ConfigurationDialogPreferredSQLEnv");
            var preferredSASWorkspace = this.findChildByID("ConfigurationDialogSASWorkspaceServerSelect");
			var impalaServerTestConnection = this.findChildByID("ConfigurationDialogImpalaServerTestConnection");
			var hiveServerTestConnection = this.findChildByID("ConfigurationDialogHiveServerTestConnection");
			//var applyButton = this.findChildByID("ConfigurationDialogHadoopConfigApply");
			
			if (this._initialShow === true) {
				this._initialShow = false;
				//cycling through all the inputs forces the invalidation warnings to show
				//when required values are missing
				//self._focusIfNotNull(dialogHost);
				this._focusIfNotNull(dialogHivePort);
				this._focusIfNotNull(dialogImpalaHost);
				this._focusIfNotNull(dialogImpalaPort);
			}

			var hiveHostHasInitialized = false;
			if (dialogHiveHost != null){
				focus.focus(dialogHiveHost);
				dialogHiveHost.set("intermediateChanges", true);
				on(dialogHiveHost, "change", function(evt) {
					if(dialogHiveHost.get("value").length > 0 && dialogHivePort.get("value").length > 0){
						//TODO: reenable test connection button when test connection is working again
						hiveServerTestConnection.set("disabled", false);
					}else{
						hiveServerTestConnection.set("disabled", true);
					}
					if(hiveHostHasInitialized){	
						self._controller.set("hiveServerConfigured", false);						
					}
					hiveHostHasInitialized=true;
				});				
			}
			
			var hivePortHasInitialized = false;
			if (dialogHivePort != null) {
				dialogHivePort.set("regExp",'[0-9]*');
				dialogHivePort.set("intermediateChanges", true);
				on(dialogHivePort, "change", function(evt) {
					if(dialogHiveHost.get("value").length > 0 && dialogHivePort.get("value").length > 0){
						//TODO: reenable test connection button when test connection is working again
						hiveServerTestConnection.set("disabled", false);
					}else{
						hiveServerTestConnection.set("disabled", true);
					}
					if(hivePortHasInitialized){
						self._controller.set("hiveServerConfigured", false);						
					}
					hivePortHasInitialized = true;
				});
			}
			
			var impalaHostHasInitialized = false;
			if (dialogImpalaHost != null){
				focus.focus(dialogImpalaHost);
				dialogImpalaHost.set("intermediateChanges", true);
				on(dialogImpalaHost, "change", function(evt) {
					if(dialogImpalaHost.get("value").length > 0  && dialogImpalaPort.get("value").length > 0){
						impalaServerTestConnection.set("disabled", false);
					}else{
						impalaServerTestConnection.set("disabled", true);
					}
					if(impalaHostHasInitialized){	
						self._controller.set("impalaServerConfigured", false);						
					}
					impalaHostHasInitialized=true;
				});				
			}
			
			var impalaPortHasInitialized = false;
			if (dialogImpalaPort != null) {
				dialogImpalaPort.set("regExp",'[0-9]*');
				dialogImpalaPort.set("intermediateChanges", true);
				on(dialogImpalaPort, "change", function(evt) {
				    if(dialogImpalaHost.get("value").length > 0  && dialogImpalaPort.get("value").length > 0){
						impalaServerTestConnection.set("disabled", false);
					}else{
						impalaServerTestConnection.set("disabled", true);
					}
					if(impalaPortHasInitialized){
						self._controller.set("impalaServerConfigured", false);						
					}
					impalaPortHasInitialized = true;
				});
			}
			
			//if (applyButton != null) {
			//	on(applyButton, "click", _lang.hitch(this, function(evt) {
			//		evt.preventDefault();
			//
			//		// First, make sure user input is valid
			//		if (this.validatePane()) {
			//			// Now, save the contents of each pane.  These all happen asynchronously.
			//			this.savePane(_lang.hitch(this, this._applySuccessHandler), _lang.hitch(this, this._applyErrorHandler));
			//		}
			//	}));
			//}
			
			if (hiveServerTestConnection != null) {
				on(hiveServerTestConnection, "click", function() {					

					var pm = self._getPresentationModelInstance();
					var server = pm.get("hadoopServer");

					var path = ResourcePathBuilder.BuildResourcePath(server);

					pm.set("isBusy", true);
					self._controller.testConnection(
							path,
							_lang.hitch(self, self._testSuccessHandler), 
							_lang.hitch(self, self._testErrorHandler, "hive", server.get("hostName"), server.get("port")))
				});
			}
			
			if (impalaServerTestConnection != null) {
				on(impalaServerTestConnection, "click", function() {

					var pm = self._getPresentationModelInstance();
					var server = pm.get("impalaServer");

					var path = ResourcePathBuilder.BuildResourcePath(server);

					pm.set("isBusy", true);
					self._controller.testConnection(
							path,
							_lang.hitch(self, self._testSuccessHandler),
							_lang.hitch(self, self._testErrorHandler, "impala", server.get("hostName"), server.get("port")))
				});
			}
		},
		
        _onGetInventoryInfoSuccess:function(result /*InventoryInfo*/) {
            this._controller.showUpdateConfigurationDialog(result, this._getPresentationModelInstance());
        },
        
        _onGetInventoryInfoError:function(error) {
            console.error(error);
        },
        
		// Override from _ConfigDialogMixin
		initializePane:function() {
			this._initialShow = true;
			if (this.__downloadComplete === false) {
				this._initializeAfterDownload = true;
			}
			else {
				this._internalInitialize();
			}
		},
		
		// Override from _ConfigDialogMixin
		validatePane:function() {
			if (this.__downloadComplete === false) {
				return true;  // This pane was never shown, so don't validate
			}
			
			this.validate();
			var currState = this.get("state");
			return (currState === "");
		},
		
		// Override from _ConfigDialogMixin
		savePane:function(successHandler/*Function*/, errorHandler/*Function*/) {
			if (this.__downloadComplete === false) {
				successHandler.call(this, {});
				return;  // This pane was never shown, so don't save anything
			}
			
			var pm = this._getPresentationModelInstance();
			var dialogHiveHost = this.findChildByID("ConfigurationDialogHiveHost");
			var dialogHivePort = this.findChildByID("ConfigurationDialogHivePort");
			var dialogImpalaHost = this.findChildByID("ConfigurationDialogImpalaHost");
			var dialogImpalaPort = this.findChildByID("ConfigurationDialogImpalaPort");
			var dialogOozie = this.findChildByID("ConfigurationDialogOozie");

			pm.set("hiveHostName", this._trimIfNotNull(dialogHiveHost.value));
			pm.set("hivePort", dialogHivePort.value);

			pm.set("impalaHostName", this._trimIfNotNull(dialogImpalaHost.value));
			var impalaPort = dialogImpalaPort.value;
			// no longer need to set this to -1 since the impala port is readonly now
			// if(StringUtils.isEmpty(impalaPort))
			// 	impalaPort = "-1";
			pm.set("impalaPort", impalaPort);
			pm.set("oozieUrl", this._trimIfNotNull(dialogOozie.value));

			pm.handleSubmit(successHandler, errorHandler);
		},
		
		_deleteCredentialsSuccessHandler:function(result){
			console.log("Delete credentials successfully");
			this._updateHiveServerConfigured();
		},
		
		_deleteCredentialsErrorHandler:function(error){
			console.error(error);
		},
		
		_applySuccessHandler:function(result){
			// the apply button does not dismiss the dialog
			// but we probably should find a way to diable the OK button while
			// the apply is pending
			// for now just log the success and move on...
			console.log("Apply returned successfully");
			this._updateHiveServerConfigured();
		},
		
		_applyErrorHandler:function(error){
			console.error(error);
		},
		
		// ** NOTE: This function was copied from _DataDirectorUIBase, because we haven't been able to figure
		// out a way to get it into a mixin so that it's available to multiple classes. **
		// override - this is a combination of the set method in Stateful and the set method in _WidgetBase
		// which allows outside classes to bind to properties this widget exposes through getters and setters
		// This essentially makes the widget act like a true Stateful when a property changes by using its setter
		set:function(name, value){
			if(typeof name === "object"){
				for(var x in name){
					if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
						this.set(x, name[x]);
					}
				}
				return this;
			}
			var names = this._getAttrNames(name),
				oldValue = this._get(name, names),
				setter = this[names.s],
				result;
			if(typeof setter === "function"){
				// use the explicit setter
				result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
			}else{
				// no setter so set attribute directly
				return this.inherited(arguments);
			}
			if(this._watchCallbacks){
				var self = this;
				// If setter returned a promise, wait for it to complete, otherwise call watches immediately
				when(result, function(){
					self._watchCallbacks(name, oldValue, self.get(name));
				});
			}
			return this; 
		},
		
		_testSuccessHandler:function(result) {
			var pm = this._getPresentationModelInstance();
			pm.set("isBusy", false);
			var msgbox = new MessageBox({title:ResourceBundle.databaseConfigurationDialog_testconnection_msgbox_title, showOkButtonOnly:true});
			msgbox.set("message", ResourceBundle.databaseConfigurationDialog_testconnection_msgbox_success);
			msgbox.show();
		},
		
		_testErrorHandler:function(server, host, port, error) {
			var pm = this._getPresentationModelInstance();
			pm.set("isBusy", false);
			var resolvedMessage = StringUtils.resolveMessage(error.get("message"), ResourceBundle);
			if (resolvedMessage == null) {
				resolvedMessage = error.get("message");
			}
			var msgbox = new MessageBox({title:ResourceBundle.databaseConfigurationDialog_testconnection_msgbox_title, showOkButtonOnly:true});
			if(resolvedMessage === null || resolvedMessage ===""){
				resolvedMessage = 
					(server === "impala") ? 
						( (host === null || host === "" || port === null || port === "") ? ResourceBundle.configurationDialog_testConnectionFailed_impalaEmptyFields_txt : ResourceBundle.configurationDialog_testConnectionFailed_default_txt)
					:ResourceBundle.configurationDialog_testConnectionFailed_default_txt;
			}
			msgbox.set("message", resolvedMessage);
			msgbox.show();
		},
		
		_unbindPresentationModel:function() {
			BindingUtils.Unbind(this._hiveServerWatcher);
			BindingUtils.Unbind(this._hiveHostNameWatcher);
			BindingUtils.Unbind(this._hivePortWatcher);
			BindingUtils.Unbind(this._impalaServerWatcher);
            BindingUtils.Unbind(this._impalaHostNameWatcher);
            BindingUtils.Unbind(this._impalaPortWatcher);
			// BindingUtils.Unbind(this._userIdWatcher);
			// BindingUtils.Unbind(this._userPasswordWatcher);
			BindingUtils.Unbind(this._errorDetailsWatcher);
			BindingUtils.Unbind(this._oozieWatcher);
			BindingUtils.Unbind(this._preferredSQLEnvironmentWatcher);
			BindingUtils.Unbind(this._preferredSQLEnvActionWatcher);
			// BindingUtils.Unbind(this._userIdHelpWatcher);
			BindingUtils.Unbind(this._busyWatcher);
			BindingUtils.Unbind(this._preferredRuntimeEnvironmentWatcher);
			BindingUtils.Unbind(this._preferredRuntimeEnvActionWatcher);
            BindingUtils.Unbind(this._preferredSASWorkspaceServerWatcher);
            BindingUtils.Unbind(this._preferredSASWorkspaceServerActionWatcher);
		},
		
		destroy:function() {
			this._unbindPresentationModel();
			var pm = this._getPresentationModelInstance();
			if (pm != null && _lang.isFunction(pm.destroy)) {
				pm.destroy();
			}
			if(this._errorDetailsDialog != null){
				this._errorDetailsDialog.destroyRecursive();
			}
		},
		
		constructor:function(params, srcNodeRef) {
			var p = params || {};
			if(p.href == null) {
				p.href = "script/js/com/sas/dm/datadirector/views/markup/HadoopConfigPane.html";
			}
			//need to check for the controller
			if (p.controller != null) {
				this._controller = p.controller;
			}
			// Manually call the super class constructor
			this.inherited(arguments,[p, srcNodeRef]);
		}
	});
	
	return HadoopConfigPane;
});
