/**
 *  This is the presentation model used to along with components/DefaultExecutionView.js
 *  and markup/DefaultExecutionView.html.  DefaultExecutionView.js binds this model to the DefaultExecutionView markup.
 *  A reference to the master controller can be acquired through the context.  The context is set on this instance
 *  from the Binder component class (DefaultExecutionView.js).  Subclassing _ContextAwareMixin ensures the  reference 
 *  to the context is stored and the _onSetContext method is called when the context becomes available.
 */

define(["dojo/_base/declare",
        "dojo/_base/lang",
        "com/sas/dm/common/components/base/_ContextAwareMixin",
        "dojo/Stateful",
        "dojo/Evented",
        "dojo/string",
        "dojo/date/locale",
        "com/sas/dm/common/util/Action",
        "com/sas/dm/common/util/BindingUtils",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/datadirector/mixins/_StepAwareMixin",
        "com/sas/dm/datadirector/models/Step",
        "com/sas/dm/datadirector/models/StepSummary",
        "com/sas/dm/datadirector/util/StepHelper",
        "com/sas/dm/datadirector/util/SharedProperties",
        "com/sas/dm/datadirector/util/ExecutionResponseActionHelper",
        "com/sas/dm/datadirector/models/ExecutionResponse",
        "com/sas/dm/common/model/ErrorResponse",
        "com/sas/dm/common/components/ErrorDialog",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], function(declare, _lang, _ContextAwareMixin, Stateful, Evented, dojoString, DateLocale, Action, BindingUtils, StringUtils, _StepAwareMixin, Step, StepSummary, StepHelper, SharedProperties, ExecutionResponseActionHelper, ExecutionResponse, ErrorResponse, ErrorDialog, resources){
	// subclass Stateful so the component class can bind to the getters and setters defined here
	// subclass _ContextAwareMixin so the component class can set a context and the _onSetContext method is called
	// subclass Evented so this class can dispatch events that the component class can hear.
	var DefaultExecutionViewPM = declare([Stateful, _ContextAwareMixin, _StepAwareMixin, Evented],{
		
		_stepStatusWatcher:null,
		_previousStepStatusWatchers:null,
		
		/*The one and only PreferencesProxy*/
		_preferencesProxy:null,
		
		/*DataDirectorController*/
		_controller:null,
		_controllerSetter: function(value/*DataDirectorController*/){
			this._controller = value;
			if(value != null){
				this._preferencesProxy = value.get("preferencesProxy");
			}
		},
		_controllerGetter: function(){
			return this._controller;
		},
		
		_viewWarnErrorAction:null,
		_viewWarnErrorActionGetter:function(){
			return this._viewWarnErrorAction;
		},
		_viewWarnErrorActionSetter:function(action/*Action*/){
			this._viewWarnErrorAction = action;
		},
		
		_executeAction:null,
		_executeActionGetter:function(){
			return this._executeAction;
		},
		_executeActionSetter:function(/*Action*/ value){
			this._executeAction = value;
		},
		
		_postExecutionActions:null,
		_postExecutionActionsGetter:function(){
			return this._postExecutionActions;
		},
		_postExecutionActionsSetter:function(/*Array of Action*/ value){
			this._postExecutionActions = value;
		},
		
		_executionStartTime:null,
		_executionStartTimeGetter:function(){
			return this._executionStartTime;
		},
		_executionStartTimeSetter:function(/*Date as String*/ value){
			this._executionStartTime = value;
		},
		
		_executionEndTime:null,
		_executionEndTimeGetter:function(){
			return this._executionEndTime;
		},
		_executionEndTimeSetter:function(/*Date as String*/ value){
			this._executionEndTime = value;
		},
		
		_executionErrorTitle:resources.defaultErrorDialog_title,
		_executionErrorTitleGetter:function(){
			return this._executionErrorTitle;
		},
		_executionErrorTitleSetter:function(/*String*/ value){
			this._executionErrorTitle = value;
		},
		
		_executionErrorDetails:null,
		_executionErrorDetailsGetter:function(){
			return this._executionErrorDetails;
		},
		_executionErrorDetailsSetter:function(/*string*/ value){
			this._executionErrorDetails = value;
			
		},
		
		_postExecutionErrorDetailsHandler:function(){
			this.set("executionErrorDetails", null);
		},
		_postExecutionErrorDetailsHandlerGetter:function(){
			return _lang.hitch(this, this._postExecutionErrorDetailsHandler);
		},
		_postExecutionErrorDetailsHandlerSetter:function(/*Function*/ value){
			this._postExecutionDetailsHandler = value;
		},
		
        _preferredSQLEnvironment:SharedProperties.preferredSqlEnvironment_default,
	    _preferredSQLEnvironmentGetter:function(){
	        return this._preferredSQLEnvironment;
	    },
	        
	    _preferredSQLEnvironmentSetter:function(value /*String*/){
	        if(value == null){
	            value = SharedProperties.preferredSqlEnvironment_default;
	        }
	                        
	        this._preferredSQLEnvironment = value;  
	    },
	    
	    _preferredSQLEnvironmentPreference:SharedProperties.preferredSqlEnvironment_default,
	    _preferredSQLEnvironmentPreferenceSetter:function(value /*String*/){
	        this._preferredSQLEnvironmentPreference = value;
	        this._updatePreferredSQLEnvironment();
	    },
	    
	    _preferredSQLEnvironmentPreferenceGetter:function(){
	        return this._preferredSQLEnvironmentPreference;
	    },
		
		_preferredExecutionEnvironment:SharedProperties.executionEnvironmentInternalName_architectSpark,
		_preferredExecutionEnvironmentGetter:function(){
			return this._preferredExecutionEnvironment;
		},
		
		_preferredExecutionEnvironmentSetter:function(value /*String*/){
			if(value == null){
				value = SharedProperties.executionEnvironmentInternalName_architectSpark;
			}
			
			this._preferredExecutionEnvironment = value;
			this._updateExecutionEnvironmentLabel();
		},		
		
		// there is a separate field for the preferred execution environment preference that we pull from 
		// user preferences.  The advanced property on the directive itself will take precedence
		// the setter and getter here are just a formality so we can use the bind method on the preferences proxy
		// for convenient change notification
		_preferredExecutionEnvironmentPreference:SharedProperties.executionEnvironmentInternalName_architectSpark,
		_preferredExecutionEnvironmentPreferenceSetter:function(value /*String*/){
			if(value == null){
				value = SharedProperties.executionEnvironmentInternalName_architectSpark;
			}
			this._preferredExecutionEnvironmentPreference = value;
			this._updatePreferredExecutionEnvironment();
		},
		
		_preferredExecutionEnvironmentPreferenceGetter:function(){
			return this._preferredExecutionEnvironmentPreference;
		},
		
		_preferredExecutionEnvironmentLabel:null,
		_preferredExecutionEnvironmentLabelGetter:function(){
			return this._preferredExecutionEnvironmentLabel;
		},
		
		_preferredExecutionEnvironmentLabelSetter:function(value /*String*/){
			this._preferredExecutionEnvironmentLabel = value;
			if(value != null){
				this._populateDefinedStepOutputs(null);
			}
		},
		
		_onPreferredExecutionEnvironmentDirectivePropertyChange:function(value /*String*/){
			// this is triggered when the user changes the advanced property for preferred runtime environment
			// on the directive itself.  Instead of saving the preference off here, just call the _updatePreferredExecutonEnvironment
			// method and let that figure out how to resolve any conflicting preferences
			this._updatePreferredExecutionEnvironment();
		},
		
		_onPreferredSQLEnvironmentDirectivePropertyChange:function(value /*String*/){
		    this._updatePreferredSQLEnvironment();
		},
		
		_updatePreferredExecutionEnvironment:function(){
			// this will set _preferredExecutionEnvironment to the correct value depending on whether the directive-level or 
			// user-level preference is set (directive-level takes precedence)
			// at the time the execution is actually kicked off, the real preferred execution environment property on the directive
			// will be populated with whatever is returned from this.get("preferredExecutionEnvironment")
			var valueToCommit = null;//SharedProperties.executionEnvironmentInternalName_default;
			var directive = this.get("controller").get("currentDirective");
			if(directive != null){
				var directiveProperties = directive.get("properties");
				var envDirectiveProp = directiveProperties.get(SharedProperties.directiveProperty_PreferredExecutionEnvironment_UI);
				// of the directive property is set to "default", use whatever is stored in user preferences instead.
				if(envDirectiveProp != SharedProperties.executionEnvironmentInternalName_global){
					valueToCommit = envDirectiveProp;
				}
			}
			if(valueToCommit == null){
				valueToCommit = this.get("preferredExecutionEnvironmentPreference");
			}
			if(valueToCommit == null){
				valueToCommit = SharedProperties.executionEnvironmentInternalName_architectSpark;
			}
			//default is always available, but if the option isn't default need to verify that the environment is actually there.
			if(valueToCommit != SharedProperties.executionEnvironmentInternalName_default){
				this.get("controller").loadExecutionEnvironments(
				    _lang.hitch(this,function(executionEnvironmentList){
				    	if(executionEnvironmentList != null){
							var executionEnvironmentModel = this._findExecutionEnvironmentInList(valueToCommit, executionEnvironmentList);
							if(valueToCommit == SharedProperties.executionEnvironmentInternalName_spark2 && executionEnvironmentModel == null){
								 var architectEnviron = this._findExecutionEnvironmentInList(SharedProperties.executionEnvironmentInternalName_architectSpark, executionEnvironmentList);
								 if(architectEnviron != null){
								 	executionEnvironmentModel = architectEnviron;
								 	valueToCommit = architectEnviron.get("internalName")
								 }
							}
							else if(valueToCommit == SharedProperties.executionEnvironmentInternalName_architectSpark && executionEnvironmentModel == null){
								var spark2Environ = this._findExecutionEnvironmentInList(SharedProperties.executionEnvironmentInternalName_spark2, executionEnvironmentList);
								if(spark2Environ != null){
									executionEnvironmentModel = spark2Environ;
									valueToCommit = spark2Environ.get("internalName")
								}
							}
							if(executionEnvironmentModel == null){
							    valueToCommit = SharedProperties.executionEnvironmentInternalName_default;
							}
						}
						this.set("preferredExecutionEnvironment", valueToCommit);
				    }),
					_lang.hitch(this,function(err){//if we can't get the available list for some reason, just carry on with what we have.
						this.set("preferredExecutionEnvironment", valueToCommit);
					})
				);

			}
			else {
				this.set("preferredExecutionEnvironment", valueToCommit);
			}
		},

		_updatePreferredSQLEnvironment:function(){
            // this will set _preferredSQLEnvironment to the correct value depending on whether the directive-level or 
            // user-level preference is set (directive-level takes precedence)
            // at the time the execution is actually kicked off, the real preferred SQL environment property on the directive
            // will be populated with whatever is returned from this.get("preferredSQLEnvironment")
            var valueToCommit = null;
            var directive = this.get("controller").get("currentDirective");
            if(directive != null){
                var directiveProperties = directive.get("properties");
                var envDirectiveProp = directiveProperties.get(SharedProperties.directiveProperty_PreferredSQLEnvironment_UI);
                // of the directive property is set to "default", use whatever is stored in user preferences instead.
                if(envDirectiveProp != SharedProperties.preferredSqlEnvironment_default){
                    valueToCommit = envDirectiveProp;
                }
            }
            if(valueToCommit == null){
                valueToCommit = this.get("preferredSQLEnvironmentPreference");
            }
            
            if(valueToCommit == null){
                valueToCommit = SharedProperties.preferredSqlEnvironment_default;
            }
            this.set("preferredSQLEnvironment", valueToCommit); 
        },		
		
		_watchUserPreference:function(destination/*String*/, key /*String*/){
            if(this._preferencesProxy != null){
                this._preferencesProxy.unbind(this, destination, key);
                this._preferencesProxy.bind(this, destination, key, true);
            }		    
		},
		
		_watchDirectiveProperty:function(handler /*property handler*/, key /*String*/){
            var controller = this.get("controller");
            if(controller != null){
                var directive = controller.get("currentDirective");
                if(directive != null){
                    var directiveProperties = directive.get("properties");
                    if(directiveProperties != null){
                        // watch the property on the directive
                        BindingUtils.HandlePropertyChange(_lang.hitch(this, handler), directiveProperties, key);
                    }
                }
                    
            }		    
		},		
        
        _useTargetEnvironment:null,
        _isTargetEnvironmentApplicableToThisDirective:function()
        {
            var controller = this.get("controller");
            
            // use the cached value if it's already been set
            if(this._useTargetEnvironment === null && controller != null) {
                this._useTargetEnvironment = controller._isAdvancedParameterInCurrentDirective(SharedProperties.directiveProperty_PreferredExecutionEnvironment_UI);   
            }
             
            return this._useTargetEnvironment;
        },
        
        _useSQLEnvironment:null,
        _isSQLEnvironmentApplicableToThisDirective:function()
        {
            var controller = this.get("controller");
            
            // use the cached value if it's already been set
            if(this._useSQLEnvironment === null && controller != null) {
                this._useSQLEnvironment = controller._isAdvancedParameterInCurrentDirective(SharedProperties.directiveProperty_PreferredSQLEnvironment_UI);   
            }
                 
            return this._useSQLEnvironment;
        },
		
		_updateExecutionEnvironmentLabel:function(){
			// we have the internalName in get("preferredExecutionEnvironment"), but we need the nls key here
			// load the full list (which should be cached - so the success handler should be called immediately) and find the correct key
			var controller = this.get("controller");
			if(controller != null){
				controller.loadExecutionEnvironments(_lang.hitch(this, this._onLoadExecutionEnvironmentListSuccess), _lang.hitch(this, this._onLoadExecutionEnvironmentListError));
			}
		},
		
		_onLoadExecutionEnvironmentListSuccess:function(executionEnvironmentList /*List of ExecutionEnvironment*/){
			var executionEnvironment = this.get("preferredExecutionEnvironment");
			if(executionEnvironment != null && executionEnvironmentList != null){
				var executionEnvironmentModel = this._findExecutionEnvironmentInList(executionEnvironment, executionEnvironmentList);
				if(executionEnvironmentModel != null){
					var nlsKey = executionEnvironmentModel.get("nlsKey");
					this._createExecutionEnvironmentLabel(nlsKey);
				}
			}
		},
		
		_createExecutionEnvironmentLabel:function(executionEnvironmentNLSKey /*String*/){
			var resolvedExecutionEnvironmentName = resources[executionEnvironmentNLSKey];
			if(resolvedExecutionEnvironmentName != null){
				var msg = dojoString.substitute(resources.executionStatus_Environment_format, [resolvedExecutionEnvironmentName]);
				console.log("step summary should be updated with "+msg);
				this.set("preferredExecutionEnvironmentLabel", msg);
			}
		},
		
		_onLoadExecutionEnvironmentListError:function(error){
			// showing the execution environment in the status pane is not critical
			// so just log this error and be done with it.
			if(error != null){
				error.resolveMessage(resources);
				console.error(error.get("message"));
			}
		},
		
		_findExecutionEnvironmentInList:function(internalName /*String*/, executionEnvironmentList /*Array of ExecutionEnvironment*/){
			// internalName is the exeuctionEnvironment name as it's stored in user preferences
			if(internalName == null || executionEnvironmentList == null){
				return null;
			}
			var executionEnvironmentListLength = executionEnvironmentList.length;
			for(var i=0; i<executionEnvironmentListLength; i++){
				var env = executionEnvironmentList[i];
				if(env != null && env.get("internalName") == internalName){
					return env;
				}
			}
			return null;
		},
		
		_executionId:null,
		
		// Override - from _ContextAwareMixin - this method is called when a context is available
		_onSetContext:function(context){
			// grab a reference to the controller from the context
			this.set("controller", context.get("Controller"));
		},
		
		// Overide - from _StepAwareMixin - called when the step is passed in through the UI
		_onStepChanged:function(step){
			var step = this.get("step");
			if(step == null){
				return;
			}
			
			var uiParams = step.get("uiParams") || {};
			var executeLabel = StringUtils.resolveString(uiParams.executionButtonLabel, resources);
			if(executeLabel == null){
				executeLabel = resources.executionStart_Default_label;
			}
			
			var executeAction = new Action({"label":executeLabel, "handler":_lang.hitch(this, this._startExecution)});
			this.set("executeAction", executeAction);
			this.set("executionErrorDetails", null);
			
			BindingUtils.Unbind(this._stepStatusWatcher);
			this._previousStepStatusWatchers.forEach(function(watcher){
				BindingUtils.Unbind(watcher);
			});
			this._previousStepStatusWatchers = [];
			
			this._stepStatusWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onStepStatusChange), step, "status");
			
			// jabosk:  some directives allow for steps to be removed without actually visiting
			// them, so it is not enough to just listen for this step's status change, but we 
			// must also listen for changes to previous steps' summary status so we know to 
			// enable/disable the run button
			var previousStep = step.get("$previousStep");
			while(previousStep != null){
				var summary = previousStep.get("stepSummary"); 
				if(summary != null){
					var previousStepType = previousStep.get("type");
					if(previousStepType == "editor"){
						this._previousStepStatusWatchers.push(BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onStepStatusChange), summary, "$progressStatus"));
					} 
				}
				previousStep = previousStep.get("$previousStep");
			}
			if(this._isTargetEnvironmentApplicableToThisDirective() == true){
			    this._watchUserPreference("preferredExecutionEnvironmentPreference", SharedProperties.userPreference_PreferredRuntimeEnvironment);
				this._watchDirectiveProperty(this._onPreferredExecutionEnvironmentDirectivePropertyChange, SharedProperties.directiveProperty_PreferredExecutionEnvironment_UI);				
			}
			if(this._isSQLEnvironmentApplicableToThisDirective() == true){			                          
                this._watchUserPreference("preferredSQLEnvironmentPreference", SharedProperties.userPreference_PreferredSQLEnvironment);
                this._watchDirectiveProperty(this._onPreferredSQLEnvironmentDirectivePropertyChange, SharedProperties.directiveProperty_PreferredSQLEnvironment_UI);	    
			}
		},
	
		_getDirectiveContext:function(){
			var context = this.get("context");
			if(context === null){ return -1;}
			
			return context.get("Controller").get("currentDirective").get("id");
		},
				
//		_completeStep:function(){	
//			this._executeAction.set("isEnabled", true);
//			this._updateStepStatus();
//			this._populateDefinedStepOutputs();
//		},
		
		
		
		_updateStepStatus:function(){
			this.get("step").set("status", Step.STATUS_COMPLETE);
		},
		
		_populateDefinedStepOutputs:function(executionStatus){
			var step = this.get("step");
			if(step == null){
				return;
			}
			var stepOutputs= this.get("step").get("definedOutputs");
			var uiParams = step.get("uiParams") || {};
			var summaryString = resources.defaultInProcessSummaryMessage_label;
			
			var stepSummary = step.get("stepSummary");
			if(stepSummary == null){
				return;
			}
			var progressStatusKey = "$progressStatus";
			var currentProgressStatus = stepSummary.get(progressStatusKey);
			if(executionStatus == null){
				// executionStatus of null typically indicates a change in the status of the runtime environment
				if(this._isTargetEnvironmentApplicableToThisDirective() == true &&
				   (currentProgressStatus == null || currentProgressStatus == StepSummary.PROGRESS_NONE || currentProgressStatus == StepSummary.PROGRESS_COMPLETE || currentProgressStatus == StepSummary.PROGRESS_ERROR)){
					summaryString = this.get("preferredExecutionEnvironmentLabel");
				}
			}else if(executionStatus == SharedProperties.executionStatus_Complete){
				stepSummary.set(progressStatusKey, StepSummary.PROGRESS_COMPLETE);
				var successSummaryUIParam = uiParams.successSummaryMessage || resources.defaultSuccessSummaryMessage_label;
				summaryString = StringUtils.resolveString(successSummaryUIParam, resources);
			}else if(this._isExecutionFailed(executionStatus)){
				stepSummary.set(progressStatusKey, StepSummary.PROGRESS_ERROR);
				var errorSummaryUIParam = uiParams.errorSummaryMessage || resources.defaultFailureSummaryMessage_label;
				summaryString = StringUtils.resolveString(errorSummaryUIParam, resources);
			}else {
				stepSummary.set(progressStatusKey, StepSummary.PROGRESS_PROCESSING);
				if(uiParams.inProcessSummaryMessage !== undefined){
					summaryString = StringUtils.resolveString(uiParams.inProcessSummaryMessage, resources);
				}
			}
			if(summaryString != null){
				stepOutputs.set(StepHelper.FindOutputKey(step, "$executionStatus"), summaryString);
			}
		},
		

	
		_processExecutionResult:function(/*ExecutionResponse*/response){
			if(response == null){
				return;
			}
			var status = response.get("status");
			// create the actions on the fly based on the values in response.get("outputs");
			var actionsToDisplay = ExecutionResponseActionHelper.GenerateActions(response.get("outputs"), this.get("controller"), this);
			
			if(status == SharedProperties.executionStatus_Error){
				//TODO make the view warnings and errors button show a stacktrace or
				// some other details about the failure.
				this.set("executionErrorTitle", resources.defaultErrorDialog_title);
				this._viewWarnErrorAction.set("handler", this._generateErrorDetailsHandler(response.get("message")));
				actionsToDisplay.unshift(this._viewWarnErrorAction);
			}

			this.set("postExecutionActions",actionsToDisplay);
			this._executeAction.set("isEnabled", true);
			this._populateDefinedStepOutputs(status);
		},
		
		
		_startExecution:function(){
			this.set("postExecutionActions", []);
			this.set("executionStartTime", null);
			this.set("executionEndTime", null);
			this._executeAction.set("isEnabled", false);
			var controller = this.get("controller");
			if(controller == null){
				return;
			}

			var directive = controller.get("currentDirective");
			this._populateDirectiveProperties(directive);
			controller.executeDirective(directive, 
										_lang.hitch(this, this._onExecutionStartSuccess),
										_lang.hitch(this, this._onExecutionStartError));
			this._populateDefinedStepOutputs(SharedProperties.executionStatus_InProgress);
										
			
			
		},
		
		_populateDirectiveProperties:function(directive /*Directive*/){
		    var directiveProperties = directive.get("properties");
		    
			// don't bother with this if target environment is not applicable...
			if(this._isTargetEnvironmentApplicableToThisDirective() == true){				
				if(directiveProperties != null){
					// set the preferredExecutionEnvironment in the directive's properties map. The value from this.get("preferredExecutionEnvironment")
					// has been distilled from from the property on the directive itself and the user preferences.
					directiveProperties.set(SharedProperties.directiveProperty_PreferredExecutionEnvironment, this.get("preferredExecutionEnvironment"));
				}else{
					console.warn("directive properties list is unexpectedly null");
				}
			}
			
            if(this._isSQLEnvironmentApplicableToThisDirective() == true){	            
	            if(directiveProperties != null){
	                // set the preferredSQLEnvironment in the directive's properties map. The value from this.get("preferredSQLEnvironment")
	                // has been distilled from from the property on the directive itself and the user preferences.
	                directiveProperties.set(SharedProperties.directiveProperty_PreferredSQLEnvironment, this.get("preferredSQLEnvironment"));
	            }else{
	                console.warn("directive properties list is unexpectedly null");
	            }
	        }
		},
		
		_onExecutionStartSuccess:function(result){
			this._populateStartDateFromResult(result);
			this._executionId = result.get("executionId");
			this._queryExecutionProgress();
		},
		//_counter:0,
		_queryExecutionProgress:function(){
			
			if(this._executionId == null){
				console.error("unable to query status for missing execution id");
				var errorResponse = new ErrorResponse();
				errorResponse.set("status", SharedProperties.executionStatus_Error);
				errorResponse.set("message", resources.executionProgressQueryFailed_txt);
				this._onExecutionProgressError(errorResponse);
				return;
			}
			var controller = this.get("controller");
			if(controller == null){
				return;
			}
			 controller.queryExectionProgress(this._executionId,
												_lang.hitch(this, this._onExecutionProgress),
												_lang.hitch(this, this._onExecutionProgressError));
		},
		_pollTimeout:null,
		_onExecutionProgress:function(result){
			// check to see if the run is in progress, succeeded or failed
			if(result == null){
				console.error("Unexpected null returned when querying execution progress");
				var errorResponse = new ErrorResponse();
				errorResponse.set("status", SharedProperties.executionStatus_Error);
				errorResponse.set("message", resources.executionProgressQueryFailed_txt);
				this._onExecutionProgressError(errorResponse);
				return;
			}
			clearTimeout(this._pollTimeout);
			var status = result.get("status");
			// if in progress set a timer and ask again later.
			this._populateStartDateFromResult(result);
			if(this._isExecutionComplete(status) == false){
//				this._generateStepActiveString("0");
				this._pollTimeout = setTimeout(_lang.hitch(this, this._queryExecutionProgress), SharedProperties.statusPollInterval);
				this._populateDefinedStepOutputs(status);
	            var actionsToDisplay = ExecutionResponseActionHelper.GenerateActions(result.get("outputs"), this.get("controller"), this);
	            this.set("postExecutionActions",actionsToDisplay);
				return;
			}else{
				this._onExecutionComplete(result);
			}

			// if failed or completed with warnings, show Warnings and Errors button
			// if has log, show Log button
			// if succeeded show results button
		},
		
		_isExecutionFailed:function(status){
			switch(status){
				case SharedProperties.executionStatus_Error:
				case SharedProperties.executionStatus_Abandoned:
				case SharedProperties.executionStatus_Stopped:
					return true;
			}
			return false;
		},
		
		_isExecutionComplete:function(status){
			if(this._isExecutionFailed(status) == true){
				return true;
			}
			switch(status){
			case SharedProperties.executionStatus_Complete:
				return true;
				
			}
			return false;
		},
		
		_onExecutionProgressError:function(error){
			// this is an error at the UIRestController level or below
			// Error message needs to indicate that job may or may not have been completed
			// determine which button need to be displayed
			//this.set("executionError", error);
			var endTime = new Date();
			this.set("executionEndTime", this._formatDate(endTime));
			if(this._executionStartTime == null){
				this.set("executionStartTime", this._formatDate(endTime));
			}
			this._populateDefinedStepOutputs(SharedProperties.executionStatus_Error);
			
			if(error.get != null && StringUtils.isEmpty(error.get("details")) == false){
				error.resolveMessage(resources);
				this._viewWarnErrorAction.set("handler", this._generateErrorResponseHandler(error));
				this.set("postExecutionActions",[this._viewWarnErrorAction]);
			}
			this._executeAction.set("isEnabled", true);
		},
		
		_onExecutionStartError:function(error){
			// an error came back when we attempted to kick off the job.
			// it may be because an exception was thrown, or because there 
			// was some http error
			var endTime = new Date();
			this.set("executionEndTime", this._formatDate(endTime));
			if(this._executionStartTime == null){
				this.set("executionStartTime", this._formatDate(endTime));
			}
			this._populateDefinedStepOutputs(SharedProperties.executionStatus_Error);
			//TODO: _viewWarnErrorActionNeeds to show the stacktrace received in the details of
			// the error
			if(error.get != null && StringUtils.isEmpty(error.get("details")) == false){
				resolvedMessage = error.resolveMessage(resources);
				this._viewWarnErrorAction.set("handler", this._generateErrorResponseHandler(error));
				this.set("postExecutionActions",[this._viewWarnErrorAction]);
			}
			this._executeAction.set("isEnabled", true);
		},
		
		_onExecutionComplete:function(result){
			console.log(result);
			var endTime = result.get("endTime") || new Date();
			this.set("executionEndTime", this._formatDate(endTime));
			if(this._executionStartTime == null){
				this.set("executionStartTime", this._formatDate(endTime));
			}
			this._processExecutionResult(result);
			
			//this._completeStep();
		},
		
		_generateErrorDetailsHandler:function(/*Array*/errorDetails){
			// show a dialog containing the error message;
			var errorMessage = "";
			if(errorDetails != null){
				if(errorDetails instanceof Array){
					var errorDetailsLength = errorDetails.length;
					for(var i=0; i<errorDetailsLength; i++){
						var errorDetail = errorDetails[i];
						if(errorDetail == null){
							continue;
						}
						var delim = "";
						if(errorMessage.length > 0){
							delim = "\n";
						}
						errorMessage = errorMessage + delim + errorDetail;
					}
				}else{
					errorMessage = errorDetails;
				}
			}
			return _lang.hitch(this, function(){
				this.set("executionErrorDetails", errorMessage);
			});
		},
		
		_generateErrorResponseHandler:function(/*ErrorResponse*/error){
			var errorDialog = new ErrorDialog({error: error});
			return _lang.hitch(this, function(){
				errorDialog.show();
			});
		},
//		_onExecutionError:function(error){
//			console.log(error);
//			
//			//this.set("executionError", error);
//			//this._completeStep();
//			//this.set("executionSuccess", error);	// TODO temp testing
//		},
		
		_populateStartDateFromResult:function(result){
			if(result == null){
				return;
			}
			var startTime = result.get("startTime");
			if(startTime != null){
				this.set("executionStartTime", this._formatDate(startTime));
			}
		},
		
		_formatDate:function(dateToFormat){
			var d = dateToFormat;
			if(d == null){
				new Date();
			}
			return DateLocale.format(d, {formatLength:'long'});
		},
		
		_onStepStatusChange:function(value/*Step.STATUS_XXX*/){
			this._disableExecutionActionIfPreviousStepsHaveWarnings();
		},
		
		_disableExecutionActionIfPreviousStepsHaveWarnings:function(){
			
			
			var executionAction = this.get("executeAction");
			if(executionAction == null){
				return;
			}
			var step = this.get("step");
			if(step == null){
				return;
			}
			var stepSummary = step.get("stepSummary");
			if(stepSummary != null){
				var progressStatusKey = "$progressStatus";
				if(stepSummary.get(progressStatusKey) == StepSummary.PROGRESS_PROCESSING){
					executionAction.set("isEnabled", false);
					return;
				}
			}
			// if any previousStep has a warning, disable the execute button
			var anyPreviousStepWarnings = false;
			var previousStep = step.get("$previousStep");
			while(previousStep != null){
				var summary = previousStep.get("stepSummary"); 
				if(summary != null){
					var status = summary.get("$progressStatus");
					var previousStepType = previousStep.get("type");
					if((previousStepType == "editor" && status != StepSummary.PROGRESS_NONE && status != StepSummary.PROGRESS_COMPLETE && status != StepSummary.PROGRESS_DISABLED && status != null)){
					//	||(previousStepType == "discovery" && previousStep.get("$dataStatus") == Step.DATA_STATUS_ERROR))
						anyPreviousStepWarnings = true;
						break;
					} 
				}
				previousStep = previousStep.get("$previousStep");
			}
			executionAction.set("isEnabled", !anyPreviousStepWarnings);
		},
				
		destroy: function(){
			// unbind stuff here
			this.inherited(arguments);
			clearTimeout(this._pollTimeout);
			if(this._preferencesProxy != null){
		 		this._preferencesProxy.unbind(this, "preferredExecutionEnvironmentPreference", SharedProperties.userPreference_PreferredRuntimeEnvironment);
		 		this._preferencesProxy.unbind(this, "preferredSQLEnvironmentPreference"      , SharedProperties.userPreference_PreferredSQLEnvironment);
			}
			BindingUtils.Unbind(this._stepStatusWatcher);
			this._previousStepStatusWatchers.forEach(function(watcher){
				BindingUtils.Unbind(watcher);
			})
		},
		
		constructor:function(){
			this._previousStepStatusWatchers = [];
			this._viewWarnErrorAction = new Action({"label":resources.executionWarnError, "iconClass":"warningsErrorsToolbarImg", "isEnabled":true, "handler":_lang.hitch(this, this._onShowErrors)});
		}
	});
	return DefaultExecutionViewPM;
});