/**
 *  This is the presentation model used to along with components/HadoopConfigPane.js
 *  and markup/HadoopConfigPane.html.  HadoopConfigPane.js binds this model to the HadoopConfigPane markup.
 *  A reference to the master controller can be acquired through the context.  The context is set on this instance
 *  from the Binder component class (HadoopConfigPane.js).  Subclassing _ContextAwareMixin ensures the  reference 
 *  to the context is stored and the _onSetContext method is called when the context becomes available.
 */

define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/config",
        "com/sas/dm/common/components/base/_ContextAwareMixin",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/common/util/ComboBoxAction",
        "com/sas/dm/common/util/OptionAction",
        "com/sas/dm/common/util/ArrayUtils",
        "com/sas/dm/datadirector/models/servers/HadoopServer",
        "com/sas/dm/datadirector/models/servers/ImpalaServer",
        "com/sas/dm/datadirector/models/SQLEnvironment",
        "com/sas/dm/datadirector/util/SharedProperties",
        "dojo/Stateful",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], function(declare, _lang, dojoConfig, _ContextAwareMixin, StringUtils, ComboBoxAction, OptionAction, ArrayUtils, HadoopServer, ImpalaServer, SQLEnvironment, SharedProperties, Stateful, resources) {

	// Subclass Stateful so the component class can bind to the getters and setters defined here
	// Subclass _ContextAwareMixin so the component class can set a context and the _onSetContext method is called
	var HadoopConfigPanePM = declare([Stateful, _ContextAwareMixin], {
		
		/*DataDirectorController*/
		_controller:null,
		_controllerSetter:function(value/*DataDirectorController*/) {
			this._controller = value;
			this._loadHadoopServerInfo();			
			this._loadImpalaServerInfo();
			this._loadPreferredSQLEnvironment();
			this._loadPreferredRuntimeEnvironment();
            this._loadPreferredSASWorkspaceServer();
		},
		_controllerGetter:function() {
			return this._controller;
		},
		
		_dialogComponent:null,
		_dialogComponentSetter:function(value/*ConfigurationDialog*/) {
			this._dialogComponent = value;
		},
		_dialogComponentGetter:function() {
			return this._dialogComponent;
		},
		
		// Override - from _ContextAwareMixin - this method is called when a context is available
		_onSetContext:function(context) {
			// Grab a reference to the controller from the context
			this.set("controller", context.get("Controller"));
		},
		
        _preferredRuntimeEnvironment:SharedProperties.executionEnvironmentInternalName_architectSpark,
	    _preferredRuntimeEnvironmentGetter:function() {
	        return this._preferredRuntimeEnvironment;
	    },
        _preferredRuntimeEnvironmentSetter:function(value /*String - SharedProperties.executionEnvironment_xxx*/) {
			if(value != null) {
				this._preferredRuntimeEnvironment = value;
			}
	    },      
	        
	    _preferredRuntimeEnvironmentAction:null,
	    _preferredRuntimeEnvironmentActionGetter:function(){
	        return this._preferredRuntimeEnvironmentAction;
	    },
	    _preferredRuntimeEnvironmentActionSetter:function(value /*ComboBoxAction*/){
	        this._preferredRuntimeEnvironmentAction = value;
	    },

        _loadPreferredRuntimeEnvironment:function() {
            if (this._controller == null) {
                return;
            }
            this.set("isBusyQueue", true);
            this._controller.getUserPreference(SharedProperties.userPreference_PreferredRuntimeEnvironment, 
                                                _lang.hitch(this, this._onPreferredRuntimeEnvironmentSuccess), 
                                                _lang.hitch(this, this._onPreferredRuntimeEnvironmentError));
        },
        
        _onPreferredRuntimeEnvironmentSuccess:function(result) {
            var resultValue = result != null ? result.value : null;
            this.set("preferredRuntimeEnvironment", resultValue);
            this._loadPreferredRuntimeEnvironmentList();
            this.set("isBusyQueue", false);
        },
        
        _onPreferredRuntimeEnvironmentError:function(error) {
            console.error(error);
            this._loadPreferredRuntimeEnvironmentList();
            this.set("isBusyQueue", false);
        },
        
        _loadPreferredRuntimeEnvironmentList:function(){
            this.set("isBusyQueue", true);
            this._controller.loadExecutionEnvironments( _lang.hitch(this, this._onLoadPreferredRuntimeEnvironmentListSuccess), 
                                                        _lang.hitch(this, this._onLoadPreferredRuntimeEnvironmentListError));
        },
       
        _onLoadPreferredRuntimeEnvironmentListSuccess:function(result /*Array of ExecutionEnvironment*/){
            if(result != null){
                this._processPreferredRuntimeEnvironmentsList(result);
            }
            this.set("isBusyQueue", false);
        },
       
        _onLoadPreferredRuntimeEnvironmentListError:function(error){
            console.error(error);
            this._generateErrorDetails(error);
            this.set("isBusyQueue", false);
        },
        
        _processPreferredRuntimeEnvironmentsList:function(executionEnvironments /*Array of ExecutionEnvironment*/){
            if(executionEnvironments != null){
                var action = this.get("preferredRuntimeEnvironmentAction");
                if(action == null){
                    action = new ComboBoxAction({"handler":_lang.hitch(this, this._onPreferredRuntimeEnvironmentChange)});
                    this.set("preferredRuntimeEnvironmentAction", action);
                }
                var optionsList = [];
                var executionEnvironmentsLength = executionEnvironments.length;
                var currentlySelectedEnvironment = this.get("preferredRuntimeEnvironment");
                for(var i=0; i<executionEnvironmentsLength; i++){
                    var environment = executionEnvironments[i];
                    if(environment != null){
                        var environmentName = environment.get("internalName");
                        var option = new OptionAction({"label":resources[environment.get("nlsKey")], "value":environmentName});
                        option.set("selected", (environmentName == currentlySelectedEnvironment));
                        optionsList.push(option);
                    }
                }
                ArrayUtils.SortOnGetter(optionsList, "label");
                action.set("dataProvider", optionsList);
            }
        },
       
        _onPreferredRuntimeEnvironmentChange:function(value, action){
            this.set("preferredRuntimeEnvironment", value);
        },

        _preferredSASWorkspaceServer:null,
        _preferredSASWorkspaceServerGetter:function() {
            return this._preferredSASWorkspaceServer;
        },
        _preferredSASWorkspaceServerSetter:function(value /*String*/) {
            if(value != null) {
                this._preferredSASWorkspaceServer = value;
            }
        },

        _preferredSASWorkspaceServerAction:null,
        _preferredSASWorkspaceServerActionGetter:function(){
            return this._preferredSASWorkspaceServerAction;
        },
        _preferredSASWorkspaceServerActionSetter:function(value /*ComboBoxAction*/){
            this._preferredSASWorkspaceServerAction = value;
        },

        _loadPreferredSASWorkspaceServer:function() {
            if (this._controller == null) {
                return;
            }
            this.set("isBusyQueue", true);
            //get the actual value from the preferences service, bypassing preferences cache/proxy so we know
			//if the value isn't coming from configuration. In future (3.2) should fold this into
			//preferences proxy, but too intrusive for defect
            this._controller.getRawUserPreference(SharedProperties.userPreference_PreferredSASWorkspaceServer,
                _lang.hitch(this, this._onPreferredSASWorkspaceServerSuccess),
                _lang.hitch(this, this._onPreferredSASWorkspaceServerError));
        },

        _onPreferredSASWorkspaceServerSuccess:function(result) {
            var resultValue = result[0] != null ? result[0]._value : null;
            this.set("preferredSASWorkspaceServer", resultValue);
            this._loadPreferredSASWorkspaceServerList();
            this.set("isBusyQueue", false);
        },

        _onPreferredSASWorkspaceServerError:function(error) {
            console.error(error);
            this._loadPreferredSASWorkspaceServerList();
            this.set("isBusyQueue", false);
        },

        _loadPreferredSASWorkspaceServerList:function(){
            this.set("isBusyQueue", true);
            this._controller.loadSASWorkspaceServers( _lang.hitch(this, this._onLoadPreferredSASWorkspaceServerListSuccess),
                _lang.hitch(this, this._onLoadPreferredSASWorkspaceServerListError));
        },

        _onLoadPreferredSASWorkspaceServerListSuccess:function(result /*Array of Strings*/){
            if(result != null){
                this._processPreferredSASWorkspaceServersList(result);
            }
            this.set("isBusyQueue", false);
        },

        _onLoadPreferredSASWorkspaceServerListError:function(error){
            console.error(error);
            this._generateErrorDetails(error);
            this.set("isBusyQueue", false);
        },

        _processPreferredSASWorkspaceServersList:function(sasWorkspaces /*Array of Strings*/){
            if(sasWorkspaces != null){
                var action = this.get("preferredSASWorkspaceServerAction");
                if(action == null){
                    action = new ComboBoxAction({"handler":_lang.hitch(this, this._onPreferredSASWorkspaceServerChange)});
                    this.set("preferredSASWorkspaceServerAction", action);
                }
                var optionsList = [];
                var currentlySelectedWorkspace = this.get("preferredSASWorkspaceServer");

                //add default option
                var defaultOption = new OptionAction({"label":resources.hadoopConfigPane_sasWorkspaceServer_option_default,"value":"(Default)"});
                defaultOption.set("selected", (currentlySelectedWorkspace == null));
                optionsList.push(defaultOption);

                var sasWorkspacesLength = sasWorkspaces.length;
                for(var i=0; i<sasWorkspacesLength; i++){
                    var sasWorkspace = sasWorkspaces[i];
                    if(sasWorkspace != null){
                        var option = new OptionAction({"label":sasWorkspace, "value":sasWorkspace});
                        option.set("selected", (sasWorkspace == currentlySelectedWorkspace));
                        optionsList.push(option);
                    }
                }
                ArrayUtils.SortOnGetter(optionsList, "label");
                action.set("dataProvider", optionsList);
            }
        },

        _onPreferredSASWorkspaceServerChange:function(value, action){
            this.set("preferredSASWorkspaceServer", value);
        },

		_impalaServer:null,
		_impalaServerGetter:function() {
			return this._impalaServer;
		},
		_impalaServerSetter:function(value /*ImpalaServer*/) {
			this._impalaServer = value;
			this.set("impalaHostName", value.hostName);
			this.set("impalaPort", value.port);
		},
		
		_hadoopServer:null,
		_hadoopServerGetter:function() {
			return this._hadoopServer;
		},
		_hadoopServerSetter:function(value /*HadoopServer*/) {
			this._hadoopServer = value;
			this.set("hiveServer", value);
			this.set("hiveHostName", value.hostName);
			this.set("hivePort", value.port);
			this.set("oozieUrl", value.get("oozieUrl"));					
		},
				
		_isBusy:false,
        _isBusyGetter:function(){
            return this._isBusy;
        },
        _isBusySetter:function(/*Boolean*/ value){
            this._isBusy = value;            
        },
        
        /*
         * Since we are using separate load data handlers, each of which resolves at different times,
         * we need to only set isBusy back to false when there are no more pending unresolved loads
         */
        _isBusyQueueUnresolvedCount: 0,
        _isBusyQueue:false,
        _isBusyQueueGetter: function(){
        	return this._isBusyQueueUnresolvedCount;
        },
        _isBusyQueueSetter: function(value /*Boolean*/){
        	if (value === true) {
        		this._isBusyQueueUnresolvedCount === 0 && this.set("isBusy", true);
        		this._isBusyQueueUnresolvedCount++;
        	} else {
        		this._isBusyQueueUnresolvedCount--;
        		this._isBusyQueueUnresolvedCount <= 0 && this.set("isBusy", false);
        	}
        },        

		_hiveHostName:null,
		_hiveHostNameGetter:function() {
			return this._hiveHostName;
		},
		_hiveHostNameSetter:function(value /*String*/) {
			this._hiveHostName = value;
		},
        
		_impalaHostName:null,
		_impalaHostNameGetter:function() {
			return this._impalaHostName;
		},
		_impalaHostNameSetter:function(value /*String*/) {
			this._impalaHostName = value;
		},

		_hivePort:null,
		_hivePortGetter:function() {
			return this._hivePort;
		},
		_hivePortSetter:function(value /*int*/) {
			this._hivePort = value;
		},
		
		_impalaPort:null,
		_impalaPortGetter:function() {
			return this._impalaPort;
		},
		_impalaPortSetter:function(value /*int*/) {
			this._impalaPort = value;
		},
		
		_oozieUrl:null,
		_oozieUrlGetter:function() {
			return this._oozieUrl;
		},
		_oozieUrlSetter:function(value /*String*/) {
			this._oozieUrl = value;
		},
		
		_userId:null,
		_userIdGetter:function() {
			return this._userId;
		},
		_userIdSetter:function(value /*String*/) {
			this._userId = value;
		},
		
		//_password:null,
		//_passwordGetter:function() {
		//	return this._password;
		//},
		//_passwordSetter:function(value /*String*/) {
		//	this._password = value;
		//},
		
		_preferredSQLEnvironment:SharedProperties.preferredSqlEnvironment_hive,
		_preferredSQLEnvironmentGetter:function() {
			return this._preferredSQLEnvironment;
		},
		_preferredSQLEnvironmentSetter:function(value /*String - SharedProperties.preferredSqlEnvironment_xxx*/) {
			if(value != null) {
				this._preferredSQLEnvironment = value;
			}
		},		
		
		_preferredSQLEnvironmentAction:null,
        _preferredSQLEnvironmentActionGetter:function(){
           return this._preferredSQLEnvironmentAction;
        },
        _preferredSQLEnvironmentActionSetter:function(value /*ComboBoxAction*/){
            this._preferredSQLEnvironmentAction = value;
        },
		
		_errorTitle:null,
		_errorTitleGetter:function() {
			return this._errorTitle;
		},
		_errorTitleSetter:function(value/*String*/) {
			this._errorTitle = value;
		},
		
		_errorDetails:null,
		_errorDetailsGetter:function() {
			return this._errorDetails;
		},
		_errorDetailsSetter:function(value/*String*/) {
			this._errorDetails = value;
		},

		_userIdHelp: null,		
		_userIdHelpSetter:function(value/*String*/){
			this._userIdHelp = value;
		},
		_userIdHelpGetter:function(){
			return this._userIdHelp;
		},
		
		_loadImpalaServerInfo:function() {
			if (this._controller == null) {
				return;
			}
			this.set("isBusyQueue", true);
			this._controller.getImpalaServer(_lang.hitch(this, this._onLoadImpalaServerSuccess), 
											this._generateErrorHandler("configurationDialog_getImpalaServer_error_txt", 
																		_lang.hitch(this, this._onLoadImpalaServerError)));
		},
		
		_onLoadImpalaServerSuccess:function(data) {
			//this should always return exactly one server			
			if (data != null && data.length > 0) {
				this.set("impalaServer", data[0]);
			}
			else {
				//this.set("impalaServer", this._createNewHadoopServer());				
			}
			this.set("isBusyQueue", false);			
		},
					
		_onLoadImpalaServerError:function(error) {
			console.error(error);
			this._generateErrorDetails(error);
			this.set("isBusyQueue", false);
		},
		       
		_loadHadoopServerInfo:function() {
			if (this._controller == null) {
				return;
			}
			this.set("isBusyQueue", true);
			this._controller.getHadoopServer(_lang.hitch(this, this._onLoadHadoopServerSuccess), this._generateErrorHandler("configurationDialog_getHadoopServer_error_txt", _lang.hitch(this, this._onLoadHadoopServerError)));
		},
		
		_onLoadHadoopServerSuccess:function(data) {
			//this should always return exactly one server
			if (data != null && data.length > 0) {
				this.set("hadoopServer", data[0]);
				//this._loadCredentials();
			}
			else {
				this.set("hadoopServer", this._createNewHadoopServer());
				//this._loadCredentials();
			}
			this._updateUserIdHelpButtonStatus();
			this.set("isBusyQueue", false);
		},
					
		_onLoadHadoopServerError:function(error) {
			console.error(error);
			this.set("hadoopServer", this._createNewHadoopServer());
			this._generateErrorDetails(error);
			this._updateUserIdHelpButtonStatus();
			this.set("isBusyQueue", false);
		},
       	
		_updateUserIdHelpButtonStatus:function(){
			var showUserIdHelpButton = false;
			if(StringUtils.isEmpty(dojoConfig.sas.commons.application.hadoop.distribution) == false){
				if(StringUtils.equalsIgnoreCase(dojoConfig.sas.commons.application.hadoop.distribution, "mapr")){
					showUserIdHelpButton = true;
				}
			}
			if(showUserIdHelpButton == true){
				this.set("userIdHelp", resources.maprUserId_help_txt)
			}else{
				this.set("userIdHelp", null);
			}
		},
		
		//_loadCredentials:function() {
		//	if (this._controller == null) {
		//		return;
		//	}
		//	var domain = this._hadoopServer.get("authDomain");
		//	this.set("isBusyQueue", true);
		//	this._controller.getCredentials(domain, _lang.hitch(this, this._onLoadCredentialsSuccess), this._generateErrorHandler("configurationDialog_getHadoopCredentials_error_txt", _lang.hitch(this, this._onLoadCredentialsError)));
		//},
		
		//_onLoadCredentialsSuccess:function(data) {
		//	var isUserIdInData = false;
		//	var userId = null;
		//	var password = null;
		//	if (data != null && data != "") {
		//		userId = data.get("principal");
		//		if(StringUtils.isEmpty(userId) == false){
		//			isUserIdInData = true;
		//		}else{
		//			userId = null;
		//		}
		//		password = data.get("credential");
		//	}
		//	if(isUserIdInData == false){
		//		// if there is no principal set, check to see if the hadoop
		//		// user system property is set and use that value to seed the user id field
		//		if(StringUtils.isEmpty(dojoConfig.sas.commons.application.hadoop.user) == false){
		//			userId = dojoConfig.sas.commons.application.hadoop.user;
		//		}
		//	}
		//	//guarantee that textbox will get cleared, i.e. changed event fired for watchers
		//	if(userId == this._userId)
		//		this._userId = userId == null ? "" : null;
		//	this.set("userId", userId);
		//	//guarantee that textbox will get cleared, i.e. changed event fired for watchers
		//	if(password == this._password)
		//		this._password = password == null ? "" : null;
		//	this.set("password", password);
		//	this.set("isBusyQueue", false);
		//},
		//
		//_onLoadCredentialsError:function(error) {
		//	console.error(error);
		//	this._generateErrorDetails(error);
		//	this.set("userId", '');
		//	this.set("password", '');
		//	this.set("isBusyQueue", false);
		//},

        _updateInventoryInfo:function(result /*InventoryInfo*/) {
            if (result != null) {
                // need to clear the values associated with text fields to 
                // make sure they are updated each time, since they may
                // have unsaved changes in them
                this.set("hiveHostName"  , null);
                this.set("hivePort"      , null);
                this.set("impalaHostName", null);
                this.set("impalaPort"    , null);                
                this.set("oozieUrl"      , null);
                
                this.set("hiveHostName"  , result._hiveHost);
                this.set("hivePort"      , result._hivePort);
                this.set("impalaHostName", result._impalaHost);
                this.set("impalaPort"    , result._impalaPort);                
                this.set("oozieUrl"      , result._oozieUrl);
                
                // dawong - 01 Sep 2015
                // S1199540 - if architect is available, set spark as the default.
                // We aren't always able to determine if spark is available, so
                // that's no longer part of this check.
                if (result._sasdmarchitectAvailable) {
                    this.set("preferredRuntimeEnvironment", SharedProperties.executionEnvironmentInternalName_architectSpark);                    
                }
                else {
                    this.set("preferredRuntimeEnvironment", SharedProperties.executionEnvironmentInternalName_default);
                }                    
                
                // if impala is available, set it as the default
                if (result._impalaAvailable) {
                    this.set("preferredSQLEnvironment", SharedProperties.preferredSqlEnvironment_impala);
                }                
                else {
                    this.set("preferredSQLEnvironment", SharedProperties.preferredSqlEnvironment_hive);
                }
            }
	    },

		_loadPreferredSQLEnvironment:function() {
			if (this._controller == null) {
				return;
			}
			this.set("isBusyQueue", true);
			this._controller.getUserPreference(SharedProperties.userPreference_PreferredSQLEnvironment, 
												_lang.hitch(this, this._onPreferredSQLEnvironmentSuccess), 
												_lang.hitch(this, this._onPreferredSQLEnvironmentError));
		},
		
		_onPreferredSQLEnvironmentSuccess:function(result) {
			var resultValue = result != null ? result.value : null;
			this.set("preferredSQLEnvironment", resultValue);
            this._loadPreferredSQLEnvironmentList();
			this.set("isBusyQueue", false);
		},
		
		_onPreferredSQLEnvironmentError:function(error) {
			console.error(error);
			this._loadPreferredSQLEnvironmentList();
			this.set("isBusyQueue", false);
		},
		
		_loadPreferredSQLEnvironmentList:function(){
        	this.set("isBusyQueue", true);
            var sqlEnvironmentList = [
                                      new SQLEnvironment({"nlsKey":"preferredSqlEnvironment_hive","internalName":SharedProperties.preferredSqlEnvironment_hive}),
                                      new SQLEnvironment({"nlsKey":"preferredSqlEnvironment_impala","internalName":SharedProperties.preferredSqlEnvironment_impala})
                                      ];
            this._processPreferredSQLEnvironmentsList(sqlEnvironmentList);     
        	this.set("isBusyQueue", false);
        },
       
        _processPreferredSQLEnvironmentsList:function(sqlEnvironments /*Array of SQLEnvironment*/){
            if(sqlEnvironments != null){
            	var action = this.get("preferredSQLEnvironmentAction");
                if(action == null){
                    action = new ComboBoxAction({"handler":_lang.hitch(this, this._onPreferredSQLEnvironmentChange)});
                    this.set("preferredSQLEnvironmentAction", action);
                }
                var optionsList = [];
                var sqlEnvironmentsLength = sqlEnvironments.length;
                var currentlySelectedEnvironment = this.get("preferredSQLEnvironment");
                for(var i=0; i<sqlEnvironmentsLength; i++){
                    var environment = sqlEnvironments[i];
                    if(environment != null){
                        var environmentName = environment.get("internalName");
                        var option = new OptionAction({"label":resources[environment.get("nlsKey")], "value":environmentName});
                        option.set("selected", (environmentName == currentlySelectedEnvironment));
                        optionsList.push(option);
                    }
                }
                ArrayUtils.SortOnGetter(optionsList, "label");
                action.set("dataProvider", optionsList);
            }
        },
       
        _onPreferredSQLEnvironmentChange:function(value, action){
            this.set("preferredSQLEnvironment", value);
		},
		
		_generateErrorDetails:function(error/*Object*/) {
			if (error == null) {
				this.set("errorTitle", null);
                this.set("errorDetails", null);
                return;
			}
			var resolvedMessage = error.resolveMessage(resources);
            var errorDetails = error.get("details");
		    var errorMessage = "";
		    if(errorDetails != null){
		        if(errorDetails instanceof Array){
		            var errorDetailsLength = errorDetails.length;
		            for(var i=0; i<errorDetailsLength; i++){
		                var errorDetail = errorDetails[i];
		                if(errorDetail == null){
		                    continue;
		                }
		                var delim = "";
		                if(errorMessage.length > 0){
		                    delim = "\n";
		                }
		                errorMessage = errorMessage + delim + errorDetail;
		            }
		        }else{
		            errorMessage = errorDetails;
		        }
		    }
		    
		    if (StringUtils.isEmpty(errorMessage) && StringUtils.isEmpty(resolvedMessage)) {
		        // No error message or details - default title, default details
	            this.set("errorTitle", resources.defaultErrorDialog_title); 
	            this.set("errorDetails", resources.defaultErrorDialog_message);
		    }
		    else if (StringUtils.isEmpty(errorMessage)) {
		        // No error message - default title, details = details
		        this.set("errorTitle", resources.defaultErrorDialog_title);
		        this.set("errorDetails", resolvedMessage);
            }
		    else if (StringUtils.isEmpty(resolvedMessage)) {
		        // No details - default title, details = error message
		        this.set("errorTitle", resources.defaultErrorDialog_title);
	            this.set("errorDetails", errorMessage);
		    }
		    else {
		        // title = error message, details = details
                this.set("errorTitle", resources.defaultErrorDialog_title);
                this.set("errorDetails", resolvedMessage+"\n"+errorMessage);
		    }
		},
		
		//deleteCredentials:function() {
		//	if (this._controller == null) {
		//		return;
		//	}
		//	var domain = this._hadoopServer.get("authDomain");
		//	this.set("isBusyQueue", true);
		//	this._controller.deleteCredentials(domain, _lang.hitch(this, this._onDeleteCredentialsSuccess), this._generateErrorHandler("configurationDialog_deleteHadoopCredentials_error_txt", _lang.hitch(this, this._onDeleteCredentialsError)));
		//},
		
		//_onDeleteCredentialsSuccess:function(data) {
		//	this._loadCredentials();
		//	this.set("isBusyQueue", false);
		//},
		
		//_onDeleteCredentialsError:function(error) {
		//	console.error(error);
		//	this._generateErrorDetails(error);
		//	this.set("isBusyQueue", false);
		//},
		
		handleSubmit:function(successHandler, errorHandler) {
			this._pendingRestCalls = 0;
			this._finalSuccessHander = successHandler;
			this._finalErrorHandler = errorHandler;
			this._anySubmitErrors = false;
			this._generateErrorDetails(null);
			
			//this._hadoopServer.set("hostName", this._hiveHostName.trim());
			//this._hadoopServer.set("port", this._hivePort);
			//this._hadoopServer.set("oozieUrl", this._oozieUrl);
			//// update impala server
			//if (this._impalaServer==null)
			//{
			//   this._impalaServer = this._createNewImpalaServer();
			//}
            //
			//this._impalaServer.set("hostName", this._impalaHostName.trim());
			//this._impalaServer.set("port", this._impalaPort);
			//this._impalaServer.set("authDomain", this._hadoopServer.get("authDomain"));
			
			//var userId = this.get("userId");
			//var password = this.get("password");
			//var domain = this._hadoopServer.get("authDomain");
			//
			////convert password to base64
			//var bytes = [];
			//for (var i = 0; i < password.length; i++) {
			//	bytes.push(password.charCodeAt(i));
			//}
			//var password_base64 = _base64.encode(bytes);

			this.set("isBusy", true);
			this._pendingRestCalls += 2;
			//this._controller.setHadoopServer(this._hadoopServer, _lang.hitch(this, this._controllerSuccessHandler), this._generateErrorHandler("configurationDialog_setHadoopServer_error_txt", _lang.hitch(this, this._controllerErrorHandler)));
			//this._controller.setCredentials(userId, password_base64, domain, _lang.hitch(this, this._controllerSuccessHandler),  this._generateErrorHandler("configurationDialog_setHadoopCredentials_error_txt", _lang.hitch(this,this._controllerErrorHandler)));
			this._controller.setUserPreference(SharedProperties.userPreference_PreferredSQLEnvironment, this.get("preferredSQLEnvironment"), _lang.hitch(this, this._controllerSuccessHandler), this._generateErrorHandler("configurationDialog_setPreferredSQLEnvironment_error_txt", _lang.hitch(this,this._controllerErrorHandler)));
			this._controller.setUserPreference(SharedProperties.userPreference_PreferredRuntimeEnvironment, this.get("preferredRuntimeEnvironment"), _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
			var workspaceServer = (this.get("preferredSASWorkspaceServer") === "(Default)") ? null : this.get("preferredSASWorkspaceServer");
            this._controller.setUserPreference(SharedProperties.userPreference_PreferredSASWorkspaceServer, workspaceServer, _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
			//if(this._impalaServer != null){
			//	++this._pendingRestCalls;
			//	this._controller.setImpalaServer(this._impalaServer, _lang.hitch(this, this._controllerSuccessHandler), this._generateErrorHandler("configurationDialog_setImpalaServer_error_txt", _lang.hitch(this,this._controllerErrorHandler)));
			//}
		},
		
		_finalSuccessHander:null,
		_finalErrorHandler:null,
		_pendingRestCalls:0,
		_anySubmitErrors:false,
		
		_generateErrorHandler: function(/*String (resource key)*/defaultErrorMessageKey, /*Function*/ errorHandler){
			return _lang.hitch(this, function(/*ErrorResponse*/error){
				if(error != null){
					if(StringUtils.isEmpty(error.resolveMessage(resources))){
						error.set("$isResolved", false);
						error.set("message", defaultErrorMessageKey);
					} 
				}
				errorHandler(error);
			});
		},
		
		_controllerSuccessHandler:function(result) {
			this._pendingRestCalls--;
			if (this._pendingRestCalls == 0 ) {
				if( !this._anySubmitErrors){
					this._finalSuccessHander(result);
				}
				this.set("isBusy", false);
			}
		},
		
		_controllerErrorHandler:function(error) {
			this._pendingRestCalls--;
			this._anySubmitErrors = true;
			this._generateErrorDetails(error);
			if(this._pendingRestCalls < 1){
				this.set("isBusy", false);
			}
		},
		
		_createNewHadoopServer:function() {
			var server = new HadoopServer();
			server.set("name", "Hadoop Server");
			server.set("id", this._createGuid());
			server.set("port", 10000);
			server.set("description", "Hadoop Server");
			server.set("authDomain", this._createAuthDomain());
			server.set("properties", {});
			server.set("paths", []);
			return server;
		},
		
		_createNewImpalaServer:function() {
			var server = new ImpalaServer();
			server.set("name", "Impala Server");
			server.set("id", this._createGuid());
			server.set("port", 21050);
			server.set("description", "Impala Server");
			server.set("authDomain", this._createAuthDomain());
			server.set("properties", {});
			server.set("paths", []);
			return server;
		},
		
		_createGuid:function() {
			return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
			    var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
			    return v.toString(16);
			});
		},
		
		_createAuthDomain:function() {
			var r = Math.random();
			var suffix = Math.floor(r*100000000);
			return "HadoopAuth_"+suffix;
		},
		
		destroy:function() {
			this.inherited(arguments);
		},
		
		constructor:function() {
			
		}
	});
	return HadoopConfigPanePM;
});
