/**
 *  This is the presentation model used to along with components/StorageSettingsConfigPane.js
 *  and markup/HadoopConfigPane.html.  StorageSettingsConfigPane.js binds this model to the StorageSettingsConfigPane markup.
 *  A reference to the master controller can be acquired through the context.  The context is set on this instance
 *  from the Binder component class (StorageSettingsConfigPane.js).  Subclassing _ContextAwareMixin ensures the  reference 
 *  to the context is stored and the _onSetContext method is called when the context becomes available.
 */

define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/config",
        "com/sas/dm/common/components/base/_ContextAwareMixin",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/common/util/BindingUtils",
		"com/sas/dm/common/util/ArrayUtils",
        "com/sas/dm/datadirector/models/servers/HadoopServer",
        "com/sas/dm/datadirector/util/SharedProperties",
        "dojo/Stateful",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], function(declare, _lang, dojoConfig, _ContextAwareMixin, StringUtils, BindingUtils, ArrayUtils, HadoopServer, SharedProperties, Stateful, resources) {
	
	// Subclass Stateful so the component class can bind to the getters and setters defined here
	// Subclass _ContextAwareMixin so the component class can set a context and the _onSetContext method is called
	var StorageSettingsConfigPanePM = declare([Stateful, _ContextAwareMixin], {
		
		/*DataDirectorController*/
		_controller:null,
		_controllerSetter:function(value/*DataDirectorController*/) {
			this._controller = value;
			this._unbindControllerWatchers();
			if(this._controller != null)
				this._bindControllerWatchers();
			this._loadHadoopServerInfo();
		},
		_controllerGetter:function() {
			return this._controller;
		},
		
		_hiveServerConfigured:false,
		_hiveServerConfiguredGetter:function(){
			return this._hiveServerConfigured;
		},
		_hiveServerConfiguredSetter:function(value/*hiveServerConfigured*/){				
			this._hiveServerConfigured = value;
		},
		
		_dialogComponent:null,
		_dialogComponentSetter:function(value/*ConfigurationDialog*/) {
			this._dialogComponent = value;
		},
		_dialogComponentGetter:function() {
			return this._dialogComponent;
		},
		
		// Override - from _ContextAwareMixin - this method is called when a context is available
		_onSetContext:function(context) {
			// Grab a reference to the controller from the context
			this.set("controller", context.get("Controller"));
		},
		
		_hadoopServer:null,
		_hadoopServerGetter:function() {
			return this._hadoopServer;
		},
		_hadoopServerSetter:function(value /*HadoopServer*/) {
			this._hadoopServer = value;
						
			var props = value.get("properties");
			if (props['temp_schema'] != null) {
				this.set("useDefaultSchema", false);
				this.set("schemaName", props['temp_schema'])
			}
			else {
				this.set("useDefaultSchema", true);
				this.set("schemaName", null)
			}
			if (props['hive_location'] != null) {
				this.set("hiveLocation", props['hive_location']);
			}
			else {
				this.set("hiveLocation", null);
			}
			if (props['lib.HDFS_TEMPDIR'] != null) {
				this.set("hdfsLocation", props['lib.HDFS_TEMPDIR']);
			}
			else {
				this.set("hdfsLocation", null);
			}			
		},
				
		_isBusy:false,
        _isBusyGetter:function(){
            return this._isBusy;
        },
        _isBusySetter:function(/*Boolean*/ value){
            this._isBusy = value;            
        },
        
        /*
         * Since we are using separate load data handlers, each of which resolves at different times,
         * we need to only set isBusy back to false when there are no more pending unresolved loads
         */
        _isBusyQueueUnresolvedCount: 0,
        _isBusyQueue:false,
        _isBusyQueueGetter: function(){
        	return this._isBusyQueueUnresolvedCount;
        },
        _isBusyQueueSetter: function(value /*Boolean*/){
        	if (value === true) {
        		this._isBusyQueueUnresolvedCount === 0 && this.set("isBusy", true);
        		this._isBusyQueueUnresolvedCount++;
        	} else {
        		this._isBusyQueueUnresolvedCount--;
        		this._isBusyQueueUnresolvedCount <= 0 && this.set("isBusy", false);
        	}
        },		
		
		_schemaName:null,
		_schemaNameGetter:function() {
			return this._schemaName;
		},
		_schemaNameSetter:function(value /*String*/) {
			this._schemaName = value;
		},
		
		_useDefaultSchema:null,
		_useDefaultSchemaGetter:function() {
			return this._useDefaultSchema;
		},
		_useDefaultSchemaSetter:function(value /*Boolean*/) {
			this._useDefaultSchema = value;
		},
		
		_hiveLocation:null,
		_hiveLocationGetter:function() {
			return this._hiveLocation;
		},
		_hiveLocationSetter:function(value /*String*/) {
			this._hiveLocation = value;
		},
		
		_hdfsLocation:null,
		_hdfsLocationGetter:function() {
			return this._hdfsLocation;
		},
		_hdfsLocationSetter:function(value /*String*/) {
			this._hdfsLocation = value;
		},
		
		_errorTitle:null,
		_errorTitleGetter:function() {
			return this._errorTitle;
		},
		_errorTitleSetter:function(value/*String*/) {
			this._errorTitle = value;
		},
		
		_errorDetails:null,
		_errorDetailsGetter:function() {
			return this._errorDetails;
		},
		_errorDetailsSetter:function(value/*String*/) {
			this._errorDetails = value;
		},
		
		_hdfsHomePath: null,
		_hdfsHomePathGetter: function(){
			return this._hdfsHomePath;
		},
		_hdfsHomePathSetter: function(value /*String*/){
			this._hdfsHomePath = value;
		},	
		
		_loadHdfsHomePathString: function() {
			// status indicator
			this.set("isBusyQueue", true);
			
			if(this.get("controller") != null ){ 
				this.get("controller").loadHdfsHomePathString(	_lang.hitch(this, this._onLoadHdfsHomePathSuccess), 
																_lang.hitch(this, this._onLoadHdfsHomePathError));			
			}
		},
		_onLoadHdfsHomePathSuccess: function(path /*String*/){
			if (path) {
				this.set("hdfsHomePath", path);
			} else {
				console.error("failed to load hdfs home path");
			}		
			this.set("isBusyQueue", false);
		},
		_onLoadHdfsHomePathError:function(error){
			this.set("error",error);
			this.set("isBusyQueue", false);
			this._generateErrorDetails(error);
		},
		
		_bindControllerWatchers:function(){
			this._hiveServerConfiguredWatcher = BindingUtils.HandlePropertyChange(	_lang.hitch(this, this._onHiveServerConfiguredChange), 
																					this._controller, "hiveServerConfigured");
		},
		
		_unbindControllerWatchers:function(){
			if(this._hiveServerConfiguredWatcher != null){
				BindingUtils.Unbind(this._hiveServerConfiguredWatcher);
				this._hiveServerConfiguredWatcher = null;
			}			
		},
		
		_onHiveServerConfiguredChange:function(val){
			this.set("hiveServerConfigured", val);
		},
		
		_loadHadoopServerInfo:function() {
			if (this._controller == null) {
				return;
			}
			this.set("isBusyQueue", true);
			this._controller.getHadoopServer(_lang.hitch(this, this._onLoadHadoopServerSuccess), 
											_lang.hitch(this, this._onLoadHadoopServerError));	
			
		},
		
		_onLoadHadoopServerSuccess:function(data) {
			//this should always return exactly one server
			if (data != null && data.length > 0) {
				this.set("hadoopServer", data[0]);				
			}
			else {
				this.set("hadoopServer", this._createNewHadoopServer());				
			}
			this.set("isBusyQueue", false);
		},
					
		_onLoadHadoopServerError:function(error) {
			console.error(error);
			this.set("hadoopServer", this._createNewHadoopServer());
			this._generateErrorDetails(error);
			this.set("isBusyQueue", false);
		},
		
		_generateErrorDetails:function(error/*Object*/) {
			if (error == null) {
				this.set("errorTitle", null);
                this.set("errorDetails", null);
                return;
			}
			var resolvedMessage = error.resolveMessage(resources);
            var errorDetails = error.get("details");
		    var errorMessage = "";
		    if(errorDetails != null){
		        if(errorDetails instanceof Array){
		            var errorDetailsLength = errorDetails.length;
		            for(var i=0; i<errorDetailsLength; i++){
		                var errorDetail = errorDetails[i];
		                if(errorDetail == null){
		                    continue;
		                }
		                var delim = "";
		                if(errorMessage.length > 0){
		                    delim = "\n";
		                }
		                errorMessage = errorMessage + delim + errorDetail;
		            }
		        }else{
		            errorMessage = errorDetails;
		        }
		    }
		    
		    if (StringUtils.isEmpty(errorMessage) && StringUtils.isEmpty(resolvedMessage)) {
		        // No error message or details - default title, default details
	            this.set("errorTitle", resources.defaultErrorDialog_title); 
	            this.set("errorDetails", resources.defaultErrorDialog_message);
		    }
		    else if (StringUtils.isEmpty(errorMessage)) {
		        // No error message - default title, details = details
		        this.set("errorTitle", resources.defaultErrorDialog_title);
		        this.set("errorDetails", resolvedMessage);
            }
		    else if (StringUtils.isEmpty(resolvedMessage)) {
		        // No details - default title, details = error message
		        this.set("errorTitle", resources.defaultErrorDialog_title);
	            this.set("errorDetails", errorMessage);
		    }
		    else {
		        // title = error message, details = details
                this.set("errorTitle", resolvedMessage);
                this.set("errorDetails", errorMessage);
		    }
		},
		
		handleSubmit:function(successHandler, errorHandler) {
			this._pendingRestCalls = 0;
			this._finalSuccessHander = successHandler;
			this._finalErrorHandler = errorHandler;
			this._anySubmitErrors = false;
			this._generateErrorDetails(null);
			
			// the hadoop server is populated with the values from user preferences when it is retrieved from the mid-tier, 
			// but the values themselves need to be set on user preferences directly and NOT through the hadoop server
			// might as well keep the local copy of the hadoop server up-to-date.
			var hiveProps = this._hadoopServer.get("properties");
			var props = {};
			if (this.get("useDefaultSchema")) {
				delete hiveProps['temp_schema'];
				props[SharedProperties.userPreference_HiveTempSchema] = null;
			}
			else {
				hiveProps['temp_schema'] = this.get("schemaName");
				props[SharedProperties.userPreference_HiveTempSchema] = this.get("schemaName");
			}
			if (this.get("hiveLocation")==null){
				delete hiveProps['hive_location'];
				props[SharedProperties.userPreference_HiveStorageLocation] = null;
			}
			else {
				hiveProps['hive_location'] = this.get("hiveLocation");
				props[SharedProperties.userPreference_HiveStorageLocation] = this.get("hiveLocation");
			}
			
			if (this.get("hdfsLocation")==null) {
				delete hiveProps['lib.HDFS_TEMPDIR'];
				props[SharedProperties.userPreference_SASHdfsTempLocation] = null;
			}
			else{
				hiveProps['lib.HDFS_TEMPDIR'] = this.get("hdfsLocation");
				props[SharedProperties.userPreference_SASHdfsTempLocation] = this.get("hdfsLocation");
			}
			
			this._hadoopServer.set("properties", hiveProps);
			var keys = ArrayUtils.GetKeys(props);
			var keysLength = keys.length;
			var buildPreferenceCallClosure = function(key, value){
				return function(){
					this._pendingRestCalls++;
					this._controller.setUserPreference(key, value,
							_lang.hitch(this, this._controllerSuccessHandler),
							_lang.hitch(this, this._controllerErrorHandler));
				};
			};
			for(var i=0; i<keysLength; i++){
				var k = keys[i];
				var callback = buildPreferenceCallClosure(k, props[k]);
				_lang.hitch(this, callback)();
			}
		},
		
		_finalSuccessHander:null,
		_finalErrorHandler:null,
		_pendingRestCalls:0,
		_anySubmitErrors:false,
		
		_controllerSuccessHandler:function(result) {
			this._pendingRestCalls--;
			if (this._pendingRestCalls == 0 && !this._anySubmitErrors) {
				this._finalSuccessHander(result);
			}
		},
		
		_controllerErrorHandler:function(error) {
			this._pendingRestCalls--;
			this._anySubmitErrors = true;
			this._generateErrorDetails(error);
		},
		
		_createNewHadoopServer:function() {
			var server = new HadoopServer();
			server.set("description", "Hadoop Server");
			server.set("properties", {});
			server.set("paths", []);
			return server;
		},
		
		_createGuid:function() {
			return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
			    var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
			    return v.toString(16);
			});
		},
		
		destroy:function() {
			this.inherited(arguments);
			this._unbindControllerWatchers();
		},
		
		constructor:function() {
			
		}
	});
	return StorageSettingsConfigPanePM;
});