define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/dom-class",
        "dijit/_WidgetBase",
        "dijit/Dialog",
        "dojo/dom-style",
        "dojo/dom-attr",
        "com/sas/dm/common/util/ComboBoxAction",
        "com/sas/dm/common/util/OptionAction",
        "com/sas/dm/common/util/ArrayUtils",
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",
        "com/sas/dm/common/components/base/_BindableWidgetBase",
        "com/sas/dm/datadirector/util/SharedProperties",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector",
        "dojo/text!./PreferredRuntimeEnvironment.html"], 
    function(declare, _lang, domClass, _WidgetBase, Dialog, domStyle, domAttr, ComboBoxAction, OptionAction, ArrayUtils, _TemplatedMixin, _WidgetsInTemplateMixin, _BindableWidgetBase, SharedProperties, resources, template) {
        var PreferredRuntimeEnvironment = declare([_WidgetBase, _BindableWidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
            templateString:template,
            nls:resources,
            props:SharedProperties,
            _defaultIsValid:true,
            _initialValue:null,

            _getValueAttr: function(){
                return this.get("preferredRuntimeEnvironment");
            },
            _setValueAttr: function(value /*String*/){
                this._initialValue = value;//could be overwritten, but only read on postCreate to avoid timing issue where value set before list items available
                this.set("preferredRuntimeEnvironment",value);
            },          
            
            _preferredRuntimeEnvironment: null,
            _setPreferredRuntimeEnvironmentAttr: function(value) 
            {
                var select = this.preferredRuntimeEnvironmentInput;
                select.set("value", value);             
            },          
            _getPreferredRuntimeEnvironmentAttr:function() 
            {
                var select = this.preferredRuntimeEnvironmentInput;
                return select.get("value");
            },
            
            postCreate:function() {
                var currentDirective = this.controller.get("currentDirective").get("type");
                var currentPreferenceDeferred = this.controller.get("preferencesProxy").get(SharedProperties.userPreference_PreferredRuntimeEnvironment);
                currentPreferenceDeferred.then(_lang.hitch(this,function(currentDefault) {
                    this._defaultIsValid = this._validForDirective(currentDefault, currentDirective);
                    this._loadPreferredRuntimeEnvironmentList();

                }), function(err){
                    console.log(err); return false;
                });
            },
            
            destroy:function(preserveDom) {
                this.inherited(arguments);
            },

            _loadPreferredRuntimeEnvironmentList:function(){
                this.controller.loadExecutionEnvironments( _lang.hitch(this, this._onLoadPreferredRuntimeEnvironmentListSuccess),
                    _lang.hitch(this, this._onLoadPreferredRuntimeEnvironmentListError));
            },

            _onLoadPreferredRuntimeEnvironmentListSuccess:function(result /*Array of ExecutionEnvironment*/){
                if(result != null){
                    this._processPreferredRuntimeEnvironmentsList(result);
                }
            },

            _onLoadPreferredRuntimeEnvironmentListError:function(error){
                console.error(error);
                this._generateErrorDetails(error);
                // this.set("isBusyQueue", false);
            },

            _onPreferredRuntimeEnvironmentChange:function(value, action){
                this.set("preferredRuntimeEnvironment", value);
            },

            _processPreferredRuntimeEnvironmentsList:function(executionEnvironments /*Array of ExecutionEnvironment*/){
                var action;
                action = new ComboBoxAction({"handler":_lang.hitch(this, this._onPreferredRuntimeEnvironmentChange)});
                this.set("preferredRuntimeEnvironmentAction", action);
                var preferredRuntimeSelect = this.preferredRuntimeEnvironmentInput;
                if(preferredRuntimeSelect != null && action != null) {
                    preferredRuntimeSelect.set("action", action);
                }
                var optionsList = [];
                if(this._defaultIsValid) {
                    var globalOption = new OptionAction({
                        "label": this.nls["executionEnvironment_global"],
                        "value": SharedProperties.executionEnvironmentInternalName_global
                    });
                    globalOption.set("selected", (globalOption.get("value") === this._initialValue));
                    optionsList.push(globalOption);
                }

                if(executionEnvironments != null){
                    var environmentOptions = [];
                    var executionEnvironmentsLength = executionEnvironments.length;
                    for(var i=0; i<executionEnvironmentsLength; i++){
                        var environment = executionEnvironments[i];
                        if(environment != null){
                            var environmentName = environment.get("internalName");
                            if(this._validForDirective(environmentName)) {
                                var option = new OptionAction({"label": this.nls[environment.get("nlsKey")], "value": environmentName});
                                option.set("selected", (environmentName === this._initialValue));
                                environmentOptions.push(option);
                            }
                        }
                    }
                    ArrayUtils.SortOnGetter(environmentOptions, "label");
                    for(var i=0; i<environmentOptions.length; i++){
                        optionsList.push(environmentOptions[i]);
                    }
                }
                action.set("dataProvider", optionsList);
            },

            _validForDirective:function(environmentName){
                //TODO should have the directive tell us what is valid, but for now just filter here because too much change to the model for hotfix.
                var currentDirective = this.controller.get("currentDirective").get("type");
                switch (environmentName) {
                    case SharedProperties.executionEnvironmentInternalName_spark2:
                        return this._supportsSpark2(currentDirective);
                    case SharedProperties.executionEnvironmentInternalName_architectSpark:
                        return this._supportsArchitectSpark(currentDirective);
                    case SharedProperties.executionEnvironmentInternalName_default:
                        return this._supportsDS2(currentDirective);
                    case SharedProperties.executionEnvironmentInternalName_global:
                        return this._defaultValid(currentDirective);
                    default:
                        return false;
                }
            },

            _supportsDS2:function(directive){
                return directive === SharedProperties.directiveType_cleanseDataInHadoop ||
                    directive === SharedProperties.directiveType_transposeDataInHadoop ||
                    directive === SharedProperties.directiveType_transformDataInHadoop ||
                    directive === SharedProperties.directiveType_mergeData;
            },

            _supportsArchitectSpark:function(directive){
                return directive === SharedProperties.directiveType_cleanseDataInHadoop ||
                    directive === SharedProperties.directiveType_transformDataInHadoop ||
                    directive === SharedProperties.directiveType_clusterSurviveInSpark;
            },

            _supportsSpark2:function(directive){
                return directive === SharedProperties.directiveType_clusterSurviveInSpark || this._supportsDS2(directive);
            },

            _defaultValid:function(currentDefaultRuntime, directive){
                if (currentDefaultRuntime === SharedProperties.executionEnvironmentInternalName_architectSpark) {
                    return this._supportsArchitectSpark(directive);
                } else if (currentDefaultRuntime === SharedProperties.executionEnvironmentInternalName_spark2) {
                    return this._supportsSpark2(directive);
                } else {
                    return this._supportsDS2(directive);
                }
            }
        });
            
        return PreferredRuntimeEnvironment
    });
