define(["dojo/_base/declare",
		"dojo/dom-class",
		"com/sas/dm/datadirector/models/Directive",
		"com/sas/dm/datadirector/models/PersistedDirective",
		"com/sas/dm/datadirector/models/PersistedDirectiveInfo",
		"com/sas/dm/datadirector/widgets/Directive",
		"com/sas/dm/common/components/ActionButton",
		"com/sas/dm/datadirector/util/IconHelper",
        "com/sas/dm/datadirector/widgets/HierarchicalButton",
        "com/sas/dm/datadirector/widgets/GridLayoutButton"],
		function(declare, domClass, Directive, PersistedDirective, PersistedDirectiveInfo, DirectiveWidget, ActionButton, IconHelper, HierarchicalButton, GridLayoutButton){
			return declare(null, {
				create:function(model /*HierarchicalObjectModel*/){
					if(model == null || model == ""){
						return null;
					}
					var type = model.get("type");
					if(type != null){
						type = type.toUpperCase();
					}
                    var isDir = model.get("isDirectory");
                    if (isDir === true) {
                        type = "FOLDER";
                    }
					switch(type){
						case "FOLDER":
						case "DIRECTORY":
						case "SYSTEMFOLDER":
						case "USERFOLDER":
							model.set("iconClass", "sasFolderIcon48_lt solidBackgroundImg");
							break;
						case "FILE":
							model.set("iconClass", "sourceFileImg noBackground");
							break;
						case "TABLE":
							var isView = model.get("isView");
							var isNew = model.get("$isNew");
							if (isView === true) {
								if (isNew === true) {
									model.set("iconClass", "sourceNewViewImg noBackground");
								}
								else {
									model.set("iconClass", "sourceViewImg noBackground");
								}
							}
							else if (isNew === true) {
								model.set("iconClass", "sourceNewTableImg noBackground");
							}
							else {
								model.set("iconClass", "sourceTableImg noBackground");
							}
							break;
						case "CATALOG":
							model.set("iconClass", "sasCatalogIcon48_lt solidBackgroundImg");
							break;
						case "SASLIBRARY":
							model.set("iconClass", "sasSourceLibraryIcon48_lt solidBackgroundImg");
							var newAction = new ActionButton({
								action: model.get("actions")["viewLibname"]
							});
							model.set("overlayBehavior", newAction);
							model.set("databaseTypeIcon", IconHelper.DatabaseTypeIcon(model.get("databaseType"), "databaseIndicatorOverlay"));
							break;
						case "DATASOURCE":
						case "DSN":
							model.set("iconClass", "sasSourceDatabaseIcon48_lt solidBackgroundImg");
							break;
						case "SCHEMA":
							model.set("iconClass", "sasDatabaseSchemaIcon48_lt solidBackgroundImg");
							break;
						case "LASR":
						case "LASRSERVER":
							model.set("iconClass", "sasLasrServerIcon48_lt solidBackgroundImg");
							break;
						case "SASWORKSPACESERVER":
						case "EXTERNALFILESERVER":
							model.set("iconClass", "sasServerIcon48_lt  solidBackgroundImg");
							break;
						case "JDBCSERVER":
						case "HADOOPSERVER":
						case "HIVEJDBCSERVER":
						case "FEDERATIONJDBCSERVER":
						case "ORACLEJDBCSERVER":
						case "TERADATAJDBCSERVER":
							model.set("iconClass", "sasSourceDatabaseIcon48_lt solidBackgroundImg");
							if(model.get("domainObject").get("properties")["connection_string"]) {
                                var newAction = new ActionButton({
                                    action: model.get("actions")["testConnection"]
                                });
                                model.set("overlayBehavior", newAction);
                            }
							model.set("databaseTypeIcon", IconHelper.DatabaseTypeIcon(model.get("databaseType"), "databaseIndicatorOverlay"));
							break;
						default:
							//check to see if it is a directive
							var directiveModel = model.get("domainObject");
							if (directiveModel instanceof Directive || directiveModel instanceof PersistedDirectiveInfo || directiveModel instanceof PersistedDirective) {
								var widget = new DirectiveWidget({"model":directiveModel, "isEnabled":true});

								// set the logo color class
								switch(directiveModel.get("category")){
									case "UTILITIES":
										domClass.add(widget.imageNode, "utilitiesColorTag");
										break;
									case "DATA INTEGRATION":
										domClass.add(widget.imageNode, "dataIntegrationColorTag");
										break;
									case "DATA FEDERATION":
										domClass.add(widget.imageNode, "dataFederationColorTag");
										break;
									case "DATA GOVERNANCE":
										domClass.add(widget.imageNode, "dataGovernanceColorTag");
										break;
									case "DATA QUALITY":
										domClass.add(widget.imageNode, "dataQualityColorTag");
										break;
									case "MASTER DATA MANAGEMENT":
										domClass.add(widget.imageNode, "masterDataManagementColorTag");
										break;
									case "DATA MOVEMENT":
										domClass.add(widget.imageNode, "dataMovementColorTag");
										break;
									default:
										domClass.add(widget.imageNode, "defaultColorTag");
								}

								return widget;
						}

						model.set("iconClass", "sourceFileImg noBackground");
					}
					
					//return new HierarchicalButton({"model":model});
					return new GridLayoutButton({"model":model});
				}
			});
})