define([
    "dojo/_base/config",
	"dojo/on",
	"dojo/query",
    "dojo/string",
	"dojo/dom",
	"dojo/_base/lang",
	"dojo/_base/declare",	         
	"dojo/dom-style",
	"dojo/dom-geometry",
	"dijit/registry",		         
	"dojo/date/locale",
	"dojo/dom-construct",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/aspect",
	"dijit/Dialog",
	"dijit/Menu",
	"dijit/MenuItem",
	"dijit/MenuSeparator",
	"dgrid/OnDemandGrid",
	"dgrid/Keyboard",
	"dgrid/extensions/ColumnResizer",	
	"dgrid/Selection",
	"dojo/keys",
	"dojo/request",
	"dojo/string",
	"dojo/io-query",
	"dojo/store/Memory",
	"dojo/topic",
	"dojo/Deferred",
	"dojo/DeferredList",
	"dojo/_base/array",
	"com/sas/dm/profile/data/ProfileData",
    "com/sas/dm/profile/common/util/SharedProperties",
	"com/sas/dm/common/components/BusyIndicator",
    "com/sas/dm/common/util/StringUtils",
    "com/sas/dm/common/util/WidthUtils",
    "com/sas/dm/common/util/UIUtils",
    "com/sas/dm/profile/common/util/GlobalizeHTML",
	"com/sas/dm/profile/common/util/dgridCaseInsensitiveSorting",
    "com/sas/dm/profile/widget/ProfileAlertWidget"
	], 
	function(baseConfig, on, query, dojoString, dom, lang, declare, domStyle, domGeom, 
			registry, locale, domConstruct, domAttr, domClass, aspect, Dialog, Menu, MenuItem, MenuSeparator,
			OnDemandGrid, Keyboard, ColumnResizer, Selection, keys, request, String, ioQuery, Memory, topic, Deferred, deferredList, arrayUtil,
			ProfileData, SharedProperties, BusyIndicator, StringUtils, WidthUtils, UIUtils, GlobalizeHTML, DgridCase, ProfileAlert){

		"use strict";
		
		// dom Node for the overall app content
		var appContent;
		
		var busyIndicator = null;
		
		//deleteCount
		var deleteCount = 0;		

		// other HTML elements (outside of the toolbar)
		var hiddenSearchStatus = {};
		
		// global dGrid object
		var	grid;
		var gridWrapper;
		var appBodyContentPane;

		// the global data controller object
		var dataHandler;

		// the globalization object
		var resourceBundle = StringUtils.getResourceMap("com.sas.dm.profile", "profile_nls");
		
		/* get references to UI elements here, these are then global */			
		var config = function() {			
									
			dataHandler = new ProfileData();
					
			appContent           = query('.appContent')[0];
			
			toolbar.self         = dom.byId('navigation');
			
			// declarative elements, get reference to widget
			toolbar.calButton    = registry.byId('calButton');
			toolbar.calendar     = registry.byId('searchCal');	
			toolbar.searchBox    = registry.byId('profileSearch');
			toolbar.deleteReportAction = registry.byId('deleteJob');

			// other elements get reference to domNode
			toolbar.calDateFilter= dom.byId('calDateFilter');
			toolbar.calLabelDate = dom.byId('dmShowAfterDate');
			toolbar.clearFilters = dom.byId('clearFilters');
			toolbar.calClearDate = dom.byId('clearDate');
			
			hiddenSearchStatus.self       = dom.byId('hiddenSearchStatus');
			hiddenSearchStatus.filterText = dom.byId("filterText");
			hiddenSearchStatus.filterResults = dom.byId('filterResults');
			
			deleteConfirmDialog.self = dom.byId('deleteConfirmDialog');
			deleteConfirmDialog.titleNode = dom.byId('deleteConfirmDialog_title');
			deleteConfirmDialog.msg = dom.byId('deleteConfirmDialog_msg');
			deleteConfirmDialog.yesButton = dom.byId('DeleteConfirmDialog_OK');
			deleteConfirmDialog.noButton = dom.byId('DeleteConfirmDialog_Cancel');
			
			grid = dom.byId('profileGrid');
			gridWrapper = dom.byId('gridWrapper');

            appBodyContentPane = registry.byId('appBody');
		};				
				
		var getDataStore = function(store) {
			
		    busyIndicator = new BusyIndicator();
		    busyIndicator.set("statusText", resourceBundle.loadingProfiles_txt);
		    busyIndicator.show(appBodyContentPane);
		    domStyle.set(busyIndicator.domNode, "display", "block");
			var profilesList =  new Deferred();
		    dataHandler.getProfileList().then(function(list){
		        profilesList.resolve(processProfileList(list, store));
		    });	
		    return profilesList;
		}
		
		var processProfileList = function(profilesList, store)
		{
			var succeededMsg = resourceBundle.succeeded_txt;
			var succeededWithErrMsg = resourceBundle.succeededWithError_txt;
			var runningMsg = resourceBundle.running_txt;
			
			var newProfilesList = [];
			
			for (var i = 0; i < profilesList.length ; i++ ){
				var item = profilesList[i];

				if (item.runDate != 0 && item.runFinished && item.finishedWithError){
					item.runStatus = succeededWithErrMsg;
				}
				else if (item.runDate != 0 && item.runFinished && !item.finishedWithError){
					item.runStatus = succeededMsg;
				}		
				else if ( item.runDate != 0 && item.runFinished == false){
					item.runStatus = runningMsg;
				} else {
					item.runStatus = "---";
				}				
				
				if ( item.runDate && item.runDate != 0 ){
					newProfilesList.push(item);
				}
			}		
						
			for (var i = 0; i < newProfilesList.length; i++) {
				var profile = newProfilesList[i];
				var queryParams = {
						jobid: profile.jobId
				};
				var url = 'profile.jsp?'+ioQuery.objectToQuery(queryParams);
				newProfilesList[i].url = url;
			}
			
			// we must define our own custom store, since the vanilla stores don't
			// support case-INsensitve sorting		
			if (store != null) {
				store.setData(newProfilesList);
			}else {
				store = new Memory({data: newProfilesList});
			}
			deleteCount = 0;
			return store;
		};

		
		/*init function, create programmatic elements here,
		or add more functionality to declaritive elements*/ 
		var initUI = function() {	
	          dijit.byId('appId').resize()//workaround for commons bug
			// set the browser tab title to also include the profile name (and capitalize the first letter)
			document.title = resourceBundle.browserTitle_txt;
		
			getDataStore().then(displayGridData);
			
		}
		
		var displayGridData = function(gridData)
		{
	         // declare custom grid to add addit'l functionality
            var CustomGrid = declare([OnDemandGrid, Keyboard, ColumnResizer, Selection], {
                    allowSelect: function(row) {
                        var rc = true;
                        if (row.data != null && row.data.runStatus != null && row.data.runStatus == "Running")
                        {
                            rc = false;
                        }
                        return rc;
                    }
                }
            );
            
			var noDataMessage = resourceBundle.noDataMessage_txt;
			grid = new CustomGrid({
				store: gridData,
				queryOptions: {
					sort: [{attribute: 'name'}],
					ignoreCase: true
				},
				minWidth: 10,
				columns: {
					name: {
						label: resourceBundle.dgridColumnName_header_title,
						formatter: function(name, object){
							if (object.runFinished == true){
								var link = urlGen(name, object.url);
								return link;
							}
							return name;
						}
					},
					location: {
						label: resourceBundle.dgridColumnPath_header_title,
						formatter: function(path, object){
							if(path){
								return path;
							}
							return object.path;
						}
					},
					runStatus: {
						label: resourceBundle.dgridColumnRunStatus_header_title,
						renderCell: statusCellRenderer
					},
					runDate: {
						label: resourceBundle.dgridColumnRunDate_header_title,
						formatter: function(version, object) {
							return dateFormatter(version, null, true);
						}
					}
				},
				//selectionMode: "multiple",
				cellNavigation: false,
				noDataMessage: noDataMessage,
				loadingMessage: resourceBundle.dgridLoading_txt
			}, grid);
			
			// disable the toolbar if no data
			if (typeof grid.noDataNode !== 'undefined') {
				toolbar.searchBox.setAttribute('disabled',true);	
				toolbar.calButton.setAttribute('disabled',true);
			}
				
			//disable (change the font-color) of not succeeded/finished profiles
			aspect.after(grid, 'renderRow', function(row, args){
				var data = args[0];
				if (data != null && data.runStatus != null && data.runStatus == "Running")
				{
					var cell = query('.dgrid-cell', row);
					cell.forEach(function(item){
						domClass.add(item,'dgridRowDisabled');
					});
				}
				return row;
			});
			
			aspect.after(grid, 'resize', function(){
			     WidthUtils.setCellTitles(this);
			});
			
							
			// setting a query triggers the aspect
			grid.set("query", {});			
			grid.startup();
    						
			// add case insensitive sorting functionality
			new DgridCase(grid);
			grid.resize();
			
			// set the button tooltips
			domAttr.set(toolbar.calButton,'title', resourceBundle.dmCalTitle_action_label);
			domAttr.set(toolbar.clearFilters,'title', resourceBundle.dmClearFilters_action_label);
			domAttr.set(toolbar.calClearDate,'title',resourceBundle.calClearDate_action_label);
			//domAttr.set(toolbar.openReportAction,'title', resourceBundle.explorerOpenAction_action_label);
			domAttr.set(toolbar.deleteReportAction,'title', resourceBundle.explorerDeleteAction_action_label);
				
			// add custom functionality to the Calendar widget
			toolbar.calendar.value = new Date();			
			addCalendarFilterLabel();			
			
			// set the place holder
			toolbar.searchBox.set('placeHolder', resourceBundle.searchPlaceHolder_txt);
	
			domStyle.set(toolbar.calClearDate, "display", 'none');
			
	         if(busyIndicator != null){
	                busyIndicator.hide(appBodyContentPane)
	            }
			
			// now bind some event handlers
			bindUIActions();			
		};
		
		var statusCellRenderer = function (object, value, node, options){
			var spanNode;			
			
			if (value == null ) {
				spanNode = domConstruct.toDom('<span class="field-runStatus-text" > '+'---'+'</span>');
			}else {
				spanNode = domConstruct.toDom('<span class="field-runStatus-text" > '+value+'</span>');
			}
			var divNode = domConstruct.create("span");
			if ( value != null && object.finishedWithError  && object.runFinished === true){
				var imageUrl = require.toUrl('sas-theme-precorporate/themes/precorporate/images/sasIcons/sasdark/16_png/ErrorCursor.png');
				var imageNode = domConstruct.toDom("<img class='field-runStatus-img' src=' '/>");
				imageNode.src = imageUrl;
				divNode.appendChild(imageNode);
			} else if ( value != null && object.runDate != 0 && object.runFinished === false) {
				var imageUrl = require.toUrl('com/sas/dm/profile/widget/images/Progress.gif');
				var imageNode = domConstruct.toDom("<img class='field-runStatus-img' src=' '/>");
				imageNode.src = imageUrl;
				divNode.appendChild(imageNode);
			}
			divNode.appendChild(spanNode);
			return divNode;
		}
		
		// checks if a profile should be disabled (from opening
		// if profile is still running
		var checkDisable = function(rowData){
			if(  rowData.runDate != 0 && rowData.runFinished === false ) 
			{
				return true;
			}
			else {
				return false;
			}
		};		
		
		/* format the date from the server */
		var dateFormatter = function(d, selector, fullYear) {
			var dt = new Date(d);
			var fy = false;
			if(fullYear){
				fy = true;
			}

			
			if (selector){
				return locale.format(dt, {selector: selector, fullYear: fy});
			}
			
			// it's formatted based on current time zone
			return locale.format(dt, {fullYear: fy});
		};
						
		/* adds a date filter label to Calendar widget. */
		/* must add as a table row */		
		var addCalendarFilterLabel = function() {
			// select the top row of the calendar table, as a ref point
			var calRefNode = query("#searchCal > table > thead > tr")[0],
				rowHTML = "<tr class='dijitReset dijitCalendarMonthContainer' valign='top'>" +
					  "<th colspan='7'></th></tr>", // create and place a new row that will hold the label
				row = domConstruct.toDom(rowHTML);

			if(calRefNode != null) {
				domConstruct.place(row, calRefNode, 'before');

				// place node in new row
				toolbar.calDateFilter.style.display = 'inline';
				calRefNode = query("#searchCal > table > thead > tr > th")[0];
				domConstruct.place(toolbar.calDateFilter, calRefNode, 'only');
			}
		};		
		
		// create a escape function
		RegExp.escape = function(s) {
		    return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&')
		};
		
		/* contains functions/methods to 
		 * handle the grid queries, and displays the queries
		 * in the status bar */ 
		var filterHandler = {						
			// return the latest values for the searchbox, calendar, and calendar's custom "clear date" button
			_getValues: function() {
				var values = {};					
				values.searchValue = toolbar.searchBox.get('value');
				values.dateValue = toolbar.calendar.get('value');	
				values.calClearDate = domStyle.get(toolbar.calClearDate, 'display');;
				if(values.calClearDate === 'none') {
					values.calClearDate = true; // hidden
				} else {
					values.calClearDate = false; // not hidden
				}
				return values;
			},
			
			// set the query to query only date
			_dateQuery: function(dateValue){
				grid.set('Query', function(object){
					return object.runDate > dateValue;
				});	
			},
			
			// set teh query to query only text
			_textQuery: function(searchValue){
				grid.set('Query', function(object){																
					var escaped = RegExp.escape(searchValue);
					var regex = new RegExp(escaped, 'i');
					return (
							( object.locationName != null  && object.locationName.match(regex) ) 
									|| ( object.name.match(regex) ) 
										/*|| ( object.path.match(regex) ) Add this back for 3.1?*/
											|| ( object.runStatus != null && object.runStatus.match(regex) )
					);
				});
			},
						
			// set the grid to query both date and text
			_queryBoth : function(tq, dq) {
				grid.set('Query', function(object){	
					var escaped = RegExp.escape(tq);
					var regex = new RegExp(escaped, 'i');				
					return ( 
							( object.runDate > dq ) &&
							((object.locationName != null && object.locationName.match(regex) ) 
								|| ( object.name.match(regex) ) 
									/*|| ( object.path.match(regex) ) Add this back for 3.1?*/
										|| ( (object.runStatus != null) && object.runStatus.match(regex) ))
					);						
				});
			},
			
			setTextFilter: function(textSearch){
				var values = this._getValues(),
					searchString;
				
	            if (values.searchValue.length === 0) {
                    this.clearTextFilter();
                    return;
                }
				
				this.setFilterDisplayText();
								
				if (values.calClearDate === false) {
					this._queryBoth(values.searchValue, values.dateValue);
				} else {
					this._textQuery(values.searchValue);
				}	
				
				console.warn(grid);
			},
			
			setDateFilter: function(){
				var values = this._getValues(),
					calDate = values.dateValue;

				this.setFilterDisplayText();
				
				if (values.searchValue.length !== 0) {
					this._queryBoth(values.searchValue, values.dateValue);
				} else {
					this._dateQuery(values.dateValue);
				}							
			},
			
			setFilterDisplayText: function(textQuery, dateQuery){
			    var values = this._getValues(),
			        dateVal = values.dateValue,
			        searchVal = values.searchValue,
			        hasSearchVal = (searchVal.length != 0),
			        hasDateVal = (values.calClearDate === false),
			        dateWithoutTime;
                
			    var filterTextContent = '';
			        
                if (hiddenSearchStatus.self.style.display !== 'inline' ) {
                    domStyle.set(hiddenSearchStatus.self, "display", 'inline');
                }
                if(hasDateVal){
                    dateWithoutTime = dateFormatter(dateVal, 'date', true);
                }

		        if(hasSearchVal && hasDateVal){
		            filterTextContent = String.substitute(resourceBundle.filterTextAndDate_txt, [searchVal, dateWithoutTime]);
                    this.setUserPreference(SharedProperties.userPreference_SavedFilterDate, dateVal.getTime() + ""); //converting to String
                    this.setUserPreference(SharedProperties.userPreference_SavedFilterText, searchVal);
		        }
		        else if (hasSearchVal){
		            filterTextContent = String.substitute(resourceBundle.filterOnlyText_txt, [searchVal]);
                    this.setUserPreference(SharedProperties.userPreference_SavedFilterText, searchVal);
		        }
		        else if(hasDateVal){             
	                filterTextContent = String.substitute(resourceBundle.filterOnlyDate_txt, [dateWithoutTime]);
                    this.setUserPreference(SharedProperties.userPreference_SavedFilterDate, dateVal.getTime() + "");//converting to String
		        }
		        else{
		            domStyle.set(hiddenSearchStatus.self, "display", 'none');
		        }
		        UIUtils.SetBasicInnerText(hiddenSearchStatus.filterText, filterTextContent);
			        
			},

            setUserPreference : function(preferenceName, value) {
                dataHandler.updatePreference(_preferencesRestUrl, preferenceName, value).then(
                    function(result){
                        //value was saved; do nothing
                    },
                    function(e){
                        //hiding errors since failing to save search filters to db doesn't need to be reported 
                    });
            },
        
			setFilterResult: function(result){
				//always have to take into account the number of deleted that are in process
				var count = parseInt(result) - deleteCount;
				var countFmtStr = (count === 1) ? resourceBundle.reportsFieldDescSingle_header_title : resourceBundle.reportsFieldDescMulti_header_title;
				var countStr = String.substitute(countFmtStr, [count]);
				hiddenSearchStatus.filterResults.innerHTML = countStr;
				
			},
			
            clearTextFilter: function(){
                this.setUserPreference(SharedProperties.userPreference_SavedFilterText, "");
                var values = this._getValues();
                
                this.setFilterDisplayText();
                
                if (values.calClearDate === false) {
                    this.setDateFilter()
                } else {
                    this.clearBoth();
                }
            },
			
			clearDateFilter: function(){
                this.setUserPreference(SharedProperties.userPreference_SavedFilterDate, "");
				var values = this._getValues();
				
				this.setFilterDisplayText();
				
				if (values.searchValue.length !== 0) {
					this.setTextFilter()
				} else {
					this.clearBoth();
				}
			},
			
			clearBoth: function() {
                this.setUserPreference(SharedProperties.userPreference_SavedFilterDate, "");
                this.setUserPreference(SharedProperties.userPreference_SavedFilterText, "");
				calendarHandler.clearCalendar();
				toolbar.searchBox.set('value','');
				this.setFilterDisplayText();
				grid.set("query", {});
			}
	
		}; // end filterHandler function
			
		/*handles event about opening job, deleting job*/ 
		var jobActionHandler = {	
				
				_getSelection:function(){
					var selectedObject = grid.selection;
					var dataStore = grid.store;
					var profileIds = [];
					var count = 0;
					for (var id in selectedObject){
						var profile = dataStore.get(id);
						if (profile) {
							profileIds[count] = profile.jobId;
							count++;
						}
					}
					return profileIds;
				},
				
				_updateDeleteSelection:function(/*List of grid IDs*/idsToRemove){
					var dataStore = grid.store;
					idsToRemove.forEach(function(id){
                        var profile = dataStore.data.forEach(function(item){
							if(item.jobId === id){
								grid.store.remove(item.id)
								deleteCount + 1;
							}
						});
					});
					grid.refresh();
				},
				
				enableActionButtons: function(){
					
					var profileIds = this._getSelection();
					
					if (profileIds && profileIds.length > 0){
						toolbar.deleteReportAction.setDisabled(false);
					}else{
						toolbar.deleteReportAction.setDisabled(true);
					}
					
				},
				
				deleteReportAction: function(){
					var profileIds = this._getSelection();
							
					if ( profileIds.length == 0) {
						//alert(resourceBundle.selectProfileAlert);
						new ProfileAlert({message: resourceBundle.selectProfileAlert_txt});
					} else if ( profileIds.length == 1 ){
						var title = resourceBundle.deleteProfile_title;
						var msg = resourceBundle.explorerDeleteAction_single_prompt_txt;
						deleteConfirmDialog.titleNode.innerHTML = title;
						deleteConfirmDialog.msg.innerHTML = msg;
						deleteConfirmDialog.show();
					} else if ( profileIds.length > 1 ){
						var title = resourceBundle.deleteProfiles_title;
						var msg = String.substitute(resourceBundle.explorerDeleteAction_multiple_prompt_txt, [profileIds.length]);
						deleteConfirmDialog.titleNode.innerHTML = title;
						deleteConfirmDialog.msg.innerHTML = msg;
						deleteConfirmDialog.show();
					}
				},
				
				deleteJobConfirmedAction: function(){
					busyIndicator = new BusyIndicator();
					busyIndicator.set("statusText", resourceBundle.deleting_txt);
					busyIndicator.show(appBodyContentPane);
					domStyle.set(busyIndicator.domNode, "display", "block");

					var selectedItems = this._getSelection();

					dataHandler.deleteJobs(selectedItems).then(
						lang.hitch(this,function(result){
							if(!result){
								result = [];
							}
							if(result.length !== selectedItems.length){
								new ProfileAlert({message: resourceBundle.explorerDeleteAction_error_txt});
							}
							this._updateDeleteSelection(result);
							busyIndicator.hide(appBodyContentPane);
						}),
						lang.hitch(this,function(err){
							new ProfileAlert({message: resourceBundle.explorerDeleteAction_error_txt});
							busyIndicator.hide(appBodyContentPane);
						})
					);
				}
				
		};
		//
		/* handles event when keys are typed in search box */
		var searchHandler = function() {			
			filterHandler.setTextFilter();					
		};
				
		// summary: 
		//			Handles all calendar events.  If no argument is given,
		//			then a calendar "date click" event is assumed and is handled
		// calClearDate: Boolean 
		//					True,false, if given, determines whether to clear the date
		// 					filter 
		//
		var calendarHandler = {};
		calendarHandler.eventHandler = function(calClearDate){	
			
			var displayDateLabel = function(){
				// set the calendar date filter label to
				// display the date without the time
				var dateWithoutTime = dateFormatter(toolbar.calendar.get('value'), 'date', true);	
				toolbar.calLabelDate.innerHTML = dateWithoutTime;
				domStyle.set(toolbar.calClearDate, 'display', 'inline');
			};
			
			if (calClearDate === false) {								
				filterHandler.setDateFilter();		
				displayDateLabel();
			}
			else if(calClearDate === true){				
				filterHandler.clearDateFilter();
				this.clearCalendar();
			}
			// date-click (date change event)
			else {										
				displayDateLabel();
				
				//var calClearDateHiddenValue = domAttr.get(toolbar.calClearDate, 'hidden');
				var calClearDateHiddenValue = domStyle.get(toolbar.calClearDate, 'display');
				// if the "clear date" button is already displayed (ie, not hidden), then trigger the date filter handle,
				// else, changing the button from true->false will auto trigger the date filter handle
				if(calClearDateHiddenValue === 'none') {
					domStyle.set(toolbar.calClearDate, 'display', 'inline');
				} else {
					this.eventHandler(false);
				}
			}			
		};
		// hides the calendar date label and the "clear" button
		calendarHandler.clearCalendar = function(){
			domStyle.set(toolbar.calClearDate, 'display', 'none');
			domAttr.set(toolbar.calLabelDate, 'innerHTML', '');
		};
		
		/* generates a url for the dgrid links */
		var urlGen = function(label, url) {
			return "<span><a href='"+url+"' class='profileLauncher'  target='_self' >"+label+"</a></span>";
		}
		/* launches selected profile in new tab */
		var profileLauncher = function(url,newTab) {
			
			if (url) {
				if(newTab === true) {
					window.open(url,'_blank')
				} else {
					window.open(url,'_self');
				}
			}
		};
				
		
		
		var refreshHandler = function() {
			filterHandler.clearBoth();
		};
				
		// Sets the margins/padding of the app content area
		var setWideOrNarrow = function(wide /*boolean*/){
			var margin;
			
			if(wide === true){
				margin = '50px';
			} else {			
				margin = '10px';
			}						
			domStyle.set(gridWrapper, 'margin', margin);			
		};
		
		/* set event actions on UI elements and stuff */
		var bindUIActions = function() {														
			/*==== Calendar events ====*/
			// calendar "clear date" button onChange event
			on(toolbar.calClearDate, 'change', function (cBoxValue){
				var calClearDateHidden = domAttr.get(toolbar.calClearDate, 'display');
				if(calClearDateHidden === 'none'){
					calendarHandler.eventHandler(true);
				} else {
					calendarHandler.eventHandler(false);
				}
				
			});
						
			// this is usually a date-change event (when a user clicks a diff date)
			toolbar.calendar.onChange = function(evt){				
				calendarHandler.eventHandler();
			};
			
			on(toolbar.calClearDate, 'click', function(){
				var hidden = domAttr.get(toolbar.calClearDate,'display');
				if(hidden === 'none'){
					domStyle.set(toolbar.calClearDate, 'display', 'none');
					calendarHandler.eventHandler(false);
					
				} else {
					domStyle.set(toolbar.calClearDate, 'display', 'inline');
					calendarHandler.eventHandler(true);
				}
				toolbar.calButton.closeDropDown();
			});
			//==== END Calendar events ====*/
			
			// update "hidden sect" when new keys are typed in the searchbox
			on(toolbar.searchBox, "change", function(){
				searchHandler();
			});
						
			// clear the search box filter, and the calendar filter when the "X" on the toolbar is clicked
			on(toolbar.clearFilters, 'click', function(evt) {
				evt.preventDefault();
				filterHandler.clearBoth();
			});
			
			// delete selected profile job
			on(deleteConfirmDialog.yesButton, 'click', function(evt){
				evt.preventDefault();
				deleteConfirmDialog.hide();
				jobActionHandler.deleteJobConfirmedAction();
			});
			
			// delete selected profile job
			on(toolbar.deleteReportAction, 'click', function(evt){
				evt.preventDefault();
				jobActionHandler.deleteReportAction();
			});
			
					
			grid.on("dgrid-refresh-complete",function(evt){
				filterHandler.setFilterResult(evt.results.total);
	             WidthUtils.setCellTitles(evt.grid);
			});
			
			grid.on(".dgrid-row:dblclick", function(evt){
				var item = grid.row(evt).data;
				if (item != null && item.runFinished == true ){
					profileLauncher(item.url, false);
				}
				console.log("dbclicked item with path:" + item.url );
			});
			
			grid.on('dgrid-error', function(evt){
				console.warn("error on grid: ", evt.error);					
			});					
			
			grid.on('dgrid-select', function(evt){
				jobActionHandler.enableActionButtons();
			});
			
			grid.on('dgrid-deselect', function(evt){
				jobActionHandler.enableActionButtons();
			});

            dataHandler.retrieveProperty(_preferencesRestUrl, SharedProperties.userPreference_SavedFilterText).then(
                lang.hitch(this, function(result){
                    if(result && result != ""){
                        toolbar.searchBox.set('value', result);
                    }
                }),
                lang.hitch(this, function(e){
                    //hiding errors since failing to save search filters to db doesn't need to be reported
                }));
            dataHandler.retrieveProperty(_preferencesRestUrl, SharedProperties.userPreference_SavedFilterDate).then(
                lang.hitch(this, function(result){
                    if(result && result != ""){
                        toolbar.calendar.set('value', new Date(parseInt(result)));
                    }
                }),
                lang.hitch(this, function(e){
                    //hiding errors since failing to save search filters to db doesn't need to be reported
                }));
		};

        var _preferencesRestUrl=null;

		var startup = function() {
			config();
			initUI();

			// replace all static HTML strings marked with class='international'
			GlobalizeHTML('international');

            _preferencesRestUrl=dojoString.substitute(SharedProperties.preferencesServiceURL,
                [encodeURIComponent(baseConfig.sas.commons.application.userDetails.userid)]);

		};
		
		/* public objects */
		var oPublic = {
			init: function() {
				startup();
			}
		};
		
		return oPublic;
		
	});
